// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: caffe_protos.proto
// Protobuf C++ Version: 7.34.1

#ifndef caffe_5fprotos_2eproto_2epb_2eh
#define caffe_5fprotos_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7034001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_caffe_5fprotos_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_caffe_5fprotos_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_caffe_5fprotos_2eproto;
}  // extern "C"
namespace caffe_protos {
enum AnnotatedDatum_AnnotationType : int;
extern const uint32_t AnnotatedDatum_AnnotationType_internal_data_[];
class AnnotatedDatum;
struct AnnotatedDatumDefaultTypeInternal;
extern AnnotatedDatumDefaultTypeInternal _AnnotatedDatum_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull AnnotatedDatum_class_data_;
class Annotation;
struct AnnotationDefaultTypeInternal;
extern AnnotationDefaultTypeInternal _Annotation_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Annotation_class_data_;
class AnnotationGroup;
struct AnnotationGroupDefaultTypeInternal;
extern AnnotationGroupDefaultTypeInternal _AnnotationGroup_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull AnnotationGroup_class_data_;
class Datum;
struct DatumDefaultTypeInternal;
extern DatumDefaultTypeInternal _Datum_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Datum_class_data_;
class NormalizedBBox;
struct NormalizedBBoxDefaultTypeInternal;
extern NormalizedBBoxDefaultTypeInternal _NormalizedBBox_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull NormalizedBBox_class_data_;
}  // namespace caffe_protos
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::caffe_protos::AnnotatedDatum_AnnotationType_internal_data_>
    internal::EnumTraitsImpl::value<::caffe_protos::AnnotatedDatum_AnnotationType>;
}  // namespace protobuf
}  // namespace google

namespace caffe_protos {
enum AnnotatedDatum_AnnotationType : int {
  AnnotatedDatum_AnnotationType_BBOX = 0,
};

extern const uint32_t AnnotatedDatum_AnnotationType_internal_data_[];
inline constexpr AnnotatedDatum_AnnotationType AnnotatedDatum_AnnotationType_AnnotationType_MIN =
    static_cast<AnnotatedDatum_AnnotationType>(0);
inline constexpr AnnotatedDatum_AnnotationType AnnotatedDatum_AnnotationType_AnnotationType_MAX =
    static_cast<AnnotatedDatum_AnnotationType>(0);
[[nodiscard]] inline bool AnnotatedDatum_AnnotationType_IsValid(int value) {
  return 0 <= value && value <= 0;
}
inline constexpr int AnnotatedDatum_AnnotationType_AnnotationType_ARRAYSIZE = 0 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
AnnotatedDatum_AnnotationType_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(AnnotatedDatum_AnnotationType) {
  return AnnotatedDatum_AnnotationType_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& AnnotatedDatum_AnnotationType_Name(T value) {
  static_assert(::std::is_same<T, AnnotatedDatum_AnnotationType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to AnnotationType_Name().");
  return AnnotatedDatum_AnnotationType_Name(static_cast<AnnotatedDatum_AnnotationType>(value));
}
template <>
[[nodiscard]] inline const ::std::string& AnnotatedDatum_AnnotationType_Name(AnnotatedDatum_AnnotationType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<AnnotatedDatum_AnnotationType_descriptor, 0, 0>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool AnnotatedDatum_AnnotationType_Parse(
    ::absl::string_view name, AnnotatedDatum_AnnotationType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<AnnotatedDatum_AnnotationType>(AnnotatedDatum_AnnotationType_descriptor(), name,
                                           value);
}

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED NormalizedBBox final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:caffe_protos.NormalizedBBox) */ {
 public:
  inline NormalizedBBox() : NormalizedBBox(nullptr) {}
  ~NormalizedBBox() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(NormalizedBBox* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(NormalizedBBox));
  }
#endif

  template <typename = void>
  explicit constexpr NormalizedBBox(::google::protobuf::internal::ConstantInitialized);

  inline NormalizedBBox(const NormalizedBBox& from) : NormalizedBBox(nullptr, from) {}
  inline NormalizedBBox(NormalizedBBox&& from) noexcept
      : NormalizedBBox(nullptr, ::std::move(from)) {}
  inline NormalizedBBox& operator=(const NormalizedBBox& from) {
    CopyFrom(from);
    return *this;
  }
  inline NormalizedBBox& operator=(NormalizedBBox&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const NormalizedBBox& default_instance() {
    return *reinterpret_cast<const NormalizedBBox*>(
        &_NormalizedBBox_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(NormalizedBBox& a, NormalizedBBox& b) { a.Swap(&b); }
  inline void Swap(NormalizedBBox* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(NormalizedBBox* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] NormalizedBBox* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<NormalizedBBox>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const NormalizedBBox& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const NormalizedBBox& from) { NormalizedBBox::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(NormalizedBBox* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "caffe_protos.NormalizedBBox"; }

  explicit NormalizedBBox(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  NormalizedBBox(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const NormalizedBBox& from);
  NormalizedBBox(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, NormalizedBBox&& from) noexcept
      : NormalizedBBox(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kXminFieldNumber = 1,
    kYminFieldNumber = 2,
    kXmaxFieldNumber = 3,
    kYmaxFieldNumber = 4,
    kLabelFieldNumber = 5,
    kDifficultFieldNumber = 6,
    kScoreFieldNumber = 7,
    kSizeFieldNumber = 8,
  };
  // optional float xmin = 1;
  [[nodiscard]] bool has_xmin()
      const;
  void clear_xmin() ;
  [[nodiscard]] float xmin() const;
  void set_xmin(float value);

  private:
  float _internal_xmin() const;
  void _internal_set_xmin(float value);

  public:
  // optional float ymin = 2;
  [[nodiscard]] bool has_ymin()
      const;
  void clear_ymin() ;
  [[nodiscard]] float ymin() const;
  void set_ymin(float value);

  private:
  float _internal_ymin() const;
  void _internal_set_ymin(float value);

  public:
  // optional float xmax = 3;
  [[nodiscard]] bool has_xmax()
      const;
  void clear_xmax() ;
  [[nodiscard]] float xmax() const;
  void set_xmax(float value);

  private:
  float _internal_xmax() const;
  void _internal_set_xmax(float value);

  public:
  // optional float ymax = 4;
  [[nodiscard]] bool has_ymax()
      const;
  void clear_ymax() ;
  [[nodiscard]] float ymax() const;
  void set_ymax(float value);

  private:
  float _internal_ymax() const;
  void _internal_set_ymax(float value);

  public:
  // optional int32 label = 5;
  [[nodiscard]] bool has_label()
      const;
  void clear_label() ;
  [[nodiscard]] ::int32_t label() const;
  void set_label(::int32_t value);

  private:
  ::int32_t _internal_label() const;
  void _internal_set_label(::int32_t value);

  public:
  // optional bool difficult = 6;
  [[nodiscard]] bool has_difficult()
      const;
  void clear_difficult() ;
  [[nodiscard]] bool difficult() const;
  void set_difficult(bool value);

  private:
  bool _internal_difficult() const;
  void _internal_set_difficult(bool value);

  public:
  // optional float score = 7;
  [[nodiscard]] bool has_score()
      const;
  void clear_score() ;
  [[nodiscard]] float score() const;
  void set_score(float value);

  private:
  float _internal_score() const;
  void _internal_set_score(float value);

  public:
  // optional float size = 8;
  [[nodiscard]] bool has_size()
      const;
  void clear_size() ;
  [[nodiscard]] float size() const;
  void set_size(float value);

  private:
  float _internal_size() const;
  void _internal_set_size(float value);

  public:
  // @@protoc_insertion_point(class_scope:caffe_protos.NormalizedBBox)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 8,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const NormalizedBBox& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    float xmin_;
    float ymin_;
    float xmax_;
    float ymax_;
    ::int32_t label_;
    bool difficult_;
    float score_;
    float size_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_caffe_5fprotos_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull NormalizedBBox_class_data_;
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Datum final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:caffe_protos.Datum) */ {
 public:
  inline Datum() : Datum(nullptr) {}
  ~Datum() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Datum* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Datum));
  }
#endif

  template <typename = void>
  explicit constexpr Datum(::google::protobuf::internal::ConstantInitialized);

  inline Datum(const Datum& from) : Datum(nullptr, from) {}
  inline Datum(Datum&& from) noexcept
      : Datum(nullptr, ::std::move(from)) {}
  inline Datum& operator=(const Datum& from) {
    CopyFrom(from);
    return *this;
  }
  inline Datum& operator=(Datum&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Datum& default_instance() {
    return *reinterpret_cast<const Datum*>(
        &_Datum_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Datum& a, Datum& b) { a.Swap(&b); }
  inline void Swap(Datum* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Datum* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Datum* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Datum>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Datum& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Datum& from) { Datum::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Datum* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "caffe_protos.Datum"; }

  explicit Datum(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Datum(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Datum& from);
  Datum(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Datum&& from) noexcept
      : Datum(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFloatDataFieldNumber = 6,
    kLabelsFieldNumber = 8,
    kDataFieldNumber = 4,
    kChannelsFieldNumber = 1,
    kHeightFieldNumber = 2,
    kWidthFieldNumber = 3,
    kLabelFieldNumber = 5,
    kEncodedFieldNumber = 7,
  };
  // repeated float float_data = 6;
  [[nodiscard]] int float_data_size()
      const;
  private:
  int _internal_float_data_size() const;

  public:
  void clear_float_data() ;
  [[nodiscard]] float float_data(int index) const;
  void set_float_data(int index, float value);
  void add_float_data(float value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<float>& float_data() const;
  ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL mutable_float_data();

  private:
  const ::google::protobuf::RepeatedField<float>& _internal_float_data() const;
  ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL _internal_mutable_float_data();

  public:
  // repeated float labels = 8;
  [[nodiscard]] int labels_size()
      const;
  private:
  int _internal_labels_size() const;

  public:
  void clear_labels() ;
  [[nodiscard]] float labels(int index) const;
  void set_labels(int index, float value);
  void add_labels(float value);
  [[nodiscard]] const ::google::protobuf::RepeatedField<float>& labels() const;
  ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL mutable_labels();

  private:
  const ::google::protobuf::RepeatedField<float>& _internal_labels() const;
  ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL _internal_mutable_labels();

  public:
  // optional bytes data = 4;
  [[nodiscard]] bool has_data()
      const;
  void clear_data() ;
  [[nodiscard]] const ::std::string& data() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_data(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_data();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_data();
  void set_allocated_data(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_data() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_data(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_data();

  public:
  // optional int32 channels = 1;
  [[nodiscard]] bool has_channels()
      const;
  void clear_channels() ;
  [[nodiscard]] ::int32_t channels() const;
  void set_channels(::int32_t value);

  private:
  ::int32_t _internal_channels() const;
  void _internal_set_channels(::int32_t value);

  public:
  // optional int32 height = 2;
  [[nodiscard]] bool has_height()
      const;
  void clear_height() ;
  [[nodiscard]] ::int32_t height() const;
  void set_height(::int32_t value);

  private:
  ::int32_t _internal_height() const;
  void _internal_set_height(::int32_t value);

  public:
  // optional int32 width = 3;
  [[nodiscard]] bool has_width()
      const;
  void clear_width() ;
  [[nodiscard]] ::int32_t width() const;
  void set_width(::int32_t value);

  private:
  ::int32_t _internal_width() const;
  void _internal_set_width(::int32_t value);

  public:
  // optional int32 label = 5;
  [[nodiscard]] bool has_label()
      const;
  void clear_label() ;
  [[nodiscard]] ::int32_t label() const;
  void set_label(::int32_t value);

  private:
  ::int32_t _internal_label() const;
  void _internal_set_label(::int32_t value);

  public:
  // optional bool encoded = 7 [default = false];
  [[nodiscard]] bool has_encoded()
      const;
  void clear_encoded() ;
  [[nodiscard]] bool encoded() const;
  void set_encoded(bool value);

  private:
  bool _internal_encoded() const;
  void _internal_set_encoded(bool value);

  public:
  // @@protoc_insertion_point(class_scope:caffe_protos.Datum)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<3, 8,
                                   0, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Datum& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedField<float> float_data_;
    ::google::protobuf::RepeatedField<float> labels_;
    ::google::protobuf::internal::ArenaStringPtr data_;
    ::int32_t channels_;
    ::int32_t height_;
    ::int32_t width_;
    ::int32_t label_;
    bool encoded_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_caffe_5fprotos_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Datum_class_data_;
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Annotation final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:caffe_protos.Annotation) */ {
 public:
  inline Annotation() : Annotation(nullptr) {}
  ~Annotation() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Annotation* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Annotation));
  }
#endif

  template <typename = void>
  explicit constexpr Annotation(::google::protobuf::internal::ConstantInitialized);

  inline Annotation(const Annotation& from) : Annotation(nullptr, from) {}
  inline Annotation(Annotation&& from) noexcept
      : Annotation(nullptr, ::std::move(from)) {}
  inline Annotation& operator=(const Annotation& from) {
    CopyFrom(from);
    return *this;
  }
  inline Annotation& operator=(Annotation&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Annotation& default_instance() {
    return *reinterpret_cast<const Annotation*>(
        &_Annotation_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(Annotation& a, Annotation& b) { a.Swap(&b); }
  inline void Swap(Annotation* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Annotation* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Annotation* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Annotation>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Annotation& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Annotation& from) { Annotation::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Annotation* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "caffe_protos.Annotation"; }

  explicit Annotation(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Annotation(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Annotation& from);
  Annotation(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Annotation&& from) noexcept
      : Annotation(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kBboxFieldNumber = 2,
    kInstanceIdFieldNumber = 1,
  };
  // optional .caffe_protos.NormalizedBBox bbox = 2;
  [[nodiscard]] bool has_bbox()
      const;
  void clear_bbox() ;
  [[nodiscard]] const ::caffe_protos::NormalizedBBox& bbox() const;
  [[nodiscard]] ::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE release_bbox();
  ::caffe_protos::NormalizedBBox* PROTOBUF_NONNULL mutable_bbox();
  void set_allocated_bbox(::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_bbox(::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE value);
  ::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE unsafe_arena_release_bbox();

  private:
  const ::caffe_protos::NormalizedBBox& _internal_bbox() const;
  ::caffe_protos::NormalizedBBox* PROTOBUF_NONNULL _internal_mutable_bbox();

  public:
  // optional int32 instance_id = 1 [default = 0];
  [[nodiscard]] bool has_instance_id()
      const;
  void clear_instance_id() ;
  [[nodiscard]] ::int32_t instance_id() const;
  void set_instance_id(::int32_t value);

  private:
  ::int32_t _internal_instance_id() const;
  void _internal_set_instance_id(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:caffe_protos.Annotation)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Annotation& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE bbox_;
    ::int32_t instance_id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_caffe_5fprotos_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Annotation_class_data_;
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED AnnotationGroup final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:caffe_protos.AnnotationGroup) */ {
 public:
  inline AnnotationGroup() : AnnotationGroup(nullptr) {}
  ~AnnotationGroup() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AnnotationGroup* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AnnotationGroup));
  }
#endif

  template <typename = void>
  explicit constexpr AnnotationGroup(::google::protobuf::internal::ConstantInitialized);

  inline AnnotationGroup(const AnnotationGroup& from) : AnnotationGroup(nullptr, from) {}
  inline AnnotationGroup(AnnotationGroup&& from) noexcept
      : AnnotationGroup(nullptr, ::std::move(from)) {}
  inline AnnotationGroup& operator=(const AnnotationGroup& from) {
    CopyFrom(from);
    return *this;
  }
  inline AnnotationGroup& operator=(AnnotationGroup&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const AnnotationGroup& default_instance() {
    return *reinterpret_cast<const AnnotationGroup*>(
        &_AnnotationGroup_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(AnnotationGroup& a, AnnotationGroup& b) { a.Swap(&b); }
  inline void Swap(AnnotationGroup* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AnnotationGroup* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] AnnotationGroup* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AnnotationGroup>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AnnotationGroup& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AnnotationGroup& from) { AnnotationGroup::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AnnotationGroup* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "caffe_protos.AnnotationGroup"; }

  explicit AnnotationGroup(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AnnotationGroup(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AnnotationGroup& from);
  AnnotationGroup(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AnnotationGroup&& from) noexcept
      : AnnotationGroup(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kAnnotationFieldNumber = 2,
    kGroupLabelFieldNumber = 1,
  };
  // repeated .caffe_protos.Annotation annotation = 2;
  [[nodiscard]] int annotation_size()
      const;
  private:
  int _internal_annotation_size() const;

  public:
  void clear_annotation() ;
  [[nodiscard]] ::caffe_protos::Annotation* PROTOBUF_NONNULL mutable_annotation(int index);
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::caffe_protos::Annotation>* PROTOBUF_NONNULL
  mutable_annotation();

  private:
  const ::google::protobuf::RepeatedPtrField<::caffe_protos::Annotation>& _internal_annotation() const;
  ::google::protobuf::RepeatedPtrField<::caffe_protos::Annotation>* PROTOBUF_NONNULL _internal_mutable_annotation();
  public:
  [[nodiscard]] const ::caffe_protos::Annotation& annotation(int index) const;
  ::caffe_protos::Annotation* PROTOBUF_NONNULL add_annotation();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::caffe_protos::Annotation>& annotation()
      const;
  // optional int32 group_label = 1;
  [[nodiscard]] bool has_group_label()
      const;
  void clear_group_label() ;
  [[nodiscard]] ::int32_t group_label() const;
  void set_group_label(::int32_t value);

  private:
  ::int32_t _internal_group_label() const;
  void _internal_set_group_label(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:caffe_protos.AnnotationGroup)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AnnotationGroup& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::caffe_protos::Annotation > annotation_;
    ::int32_t group_label_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_caffe_5fprotos_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull AnnotationGroup_class_data_;
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED AnnotatedDatum final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:caffe_protos.AnnotatedDatum) */ {
 public:
  inline AnnotatedDatum() : AnnotatedDatum(nullptr) {}
  ~AnnotatedDatum() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(AnnotatedDatum* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(AnnotatedDatum));
  }
#endif

  template <typename = void>
  explicit constexpr AnnotatedDatum(::google::protobuf::internal::ConstantInitialized);

  inline AnnotatedDatum(const AnnotatedDatum& from) : AnnotatedDatum(nullptr, from) {}
  inline AnnotatedDatum(AnnotatedDatum&& from) noexcept
      : AnnotatedDatum(nullptr, ::std::move(from)) {}
  inline AnnotatedDatum& operator=(const AnnotatedDatum& from) {
    CopyFrom(from);
    return *this;
  }
  inline AnnotatedDatum& operator=(AnnotatedDatum&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const AnnotatedDatum& default_instance() {
    return *reinterpret_cast<const AnnotatedDatum*>(
        &_AnnotatedDatum_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(AnnotatedDatum& a, AnnotatedDatum& b) { a.Swap(&b); }
  inline void Swap(AnnotatedDatum* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(AnnotatedDatum* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] AnnotatedDatum* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<AnnotatedDatum>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const AnnotatedDatum& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const AnnotatedDatum& from) { AnnotatedDatum::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(AnnotatedDatum* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "caffe_protos.AnnotatedDatum"; }

  explicit AnnotatedDatum(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  AnnotatedDatum(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const AnnotatedDatum& from);
  AnnotatedDatum(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, AnnotatedDatum&& from) noexcept
      : AnnotatedDatum(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------
  using AnnotationType = AnnotatedDatum_AnnotationType;
  static constexpr AnnotationType BBOX = AnnotatedDatum_AnnotationType_BBOX;
  [[nodiscard]] static inline bool AnnotationType_IsValid(int value) {
    return AnnotatedDatum_AnnotationType_IsValid(value);
  }
  static constexpr AnnotationType AnnotationType_MIN = AnnotatedDatum_AnnotationType_AnnotationType_MIN;
  static constexpr AnnotationType AnnotationType_MAX = AnnotatedDatum_AnnotationType_AnnotationType_MAX;
  static constexpr int AnnotationType_ARRAYSIZE = AnnotatedDatum_AnnotationType_AnnotationType_ARRAYSIZE;
  static inline const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL AnnotationType_descriptor() {
    return AnnotatedDatum_AnnotationType_descriptor();
  }
  template <typename T>
  [[nodiscard]] static inline const ::std::string& AnnotationType_Name(T value) {
    return AnnotatedDatum_AnnotationType_Name(value);
  }
  [[nodiscard]] static inline bool AnnotationType_Parse(
      ::absl::string_view name, AnnotationType* PROTOBUF_NONNULL value) {
    return AnnotatedDatum_AnnotationType_Parse(name, value);
  }

  // accessors -------------------------------------------------------
  enum : int {
    kAnnotationGroupFieldNumber = 3,
    kDatumFieldNumber = 1,
    kTypeFieldNumber = 2,
  };
  // repeated .caffe_protos.AnnotationGroup annotation_group = 3;
  [[nodiscard]] int annotation_group_size()
      const;
  private:
  int _internal_annotation_group_size() const;

  public:
  void clear_annotation_group() ;
  [[nodiscard]] ::caffe_protos::AnnotationGroup* PROTOBUF_NONNULL mutable_annotation_group(int index);
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::caffe_protos::AnnotationGroup>* PROTOBUF_NONNULL
  mutable_annotation_group();

  private:
  const ::google::protobuf::RepeatedPtrField<::caffe_protos::AnnotationGroup>& _internal_annotation_group() const;
  ::google::protobuf::RepeatedPtrField<::caffe_protos::AnnotationGroup>* PROTOBUF_NONNULL _internal_mutable_annotation_group();
  public:
  [[nodiscard]] const ::caffe_protos::AnnotationGroup& annotation_group(int index) const;
  ::caffe_protos::AnnotationGroup* PROTOBUF_NONNULL add_annotation_group();
  [[nodiscard]] const ::google::protobuf::RepeatedPtrField<::caffe_protos::AnnotationGroup>& annotation_group()
      const;
  // optional .caffe_protos.Datum datum = 1;
  [[nodiscard]] bool has_datum()
      const;
  void clear_datum() ;
  [[nodiscard]] const ::caffe_protos::Datum& datum() const;
  [[nodiscard]] ::caffe_protos::Datum* PROTOBUF_NULLABLE release_datum();
  ::caffe_protos::Datum* PROTOBUF_NONNULL mutable_datum();
  void set_allocated_datum(::caffe_protos::Datum* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_datum(::caffe_protos::Datum* PROTOBUF_NULLABLE value);
  ::caffe_protos::Datum* PROTOBUF_NULLABLE unsafe_arena_release_datum();

  private:
  const ::caffe_protos::Datum& _internal_datum() const;
  ::caffe_protos::Datum* PROTOBUF_NONNULL _internal_mutable_datum();

  public:
  // optional .caffe_protos.AnnotatedDatum.AnnotationType type = 2;
  [[nodiscard]] bool has_type()
      const;
  void clear_type() ;
  [[nodiscard]] ::caffe_protos::AnnotatedDatum_AnnotationType type() const;
  void set_type(::caffe_protos::AnnotatedDatum_AnnotationType value);

  private:
  ::caffe_protos::AnnotatedDatum_AnnotationType _internal_type() const;
  void _internal_set_type(::caffe_protos::AnnotatedDatum_AnnotationType value);

  public:
  // @@protoc_insertion_point(class_scope:caffe_protos.AnnotatedDatum)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<2, 3,
                                   3, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const AnnotatedDatum& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField< ::caffe_protos::AnnotationGroup > annotation_group_;
    ::caffe_protos::Datum* PROTOBUF_NULLABLE datum_;
    int type_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_caffe_5fprotos_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull AnnotatedDatum_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Datum

// optional int32 channels = 1;
inline bool Datum::has_channels() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void Datum::clear_channels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.channels_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline ::int32_t Datum::channels() const {
  // @@protoc_insertion_point(field_get:caffe_protos.Datum.channels)
  return _internal_channels();
}
inline void Datum::set_channels(::int32_t value) {
  _internal_set_channels(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:caffe_protos.Datum.channels)
}
inline ::int32_t Datum::_internal_channels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.channels_;
}
inline void Datum::_internal_set_channels(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.channels_ = value;
}

// optional int32 height = 2;
inline bool Datum::has_height() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void Datum::clear_height() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.height_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline ::int32_t Datum::height() const {
  // @@protoc_insertion_point(field_get:caffe_protos.Datum.height)
  return _internal_height();
}
inline void Datum::set_height(::int32_t value) {
  _internal_set_height(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:caffe_protos.Datum.height)
}
inline ::int32_t Datum::_internal_height() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.height_;
}
inline void Datum::_internal_set_height(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.height_ = value;
}

// optional int32 width = 3;
inline bool Datum::has_width() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  return value;
}
inline void Datum::clear_width() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.width_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline ::int32_t Datum::width() const {
  // @@protoc_insertion_point(field_get:caffe_protos.Datum.width)
  return _internal_width();
}
inline void Datum::set_width(::int32_t value) {
  _internal_set_width(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:caffe_protos.Datum.width)
}
inline ::int32_t Datum::_internal_width() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.width_;
}
inline void Datum::_internal_set_width(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.width_ = value;
}

// optional bytes data = 4;
inline bool Datum::has_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void Datum::clear_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline const ::std::string& Datum::data() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:caffe_protos.Datum.data)
  return _internal_data();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Datum::set_data(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.data_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:caffe_protos.Datum.data)
}
inline ::std::string* PROTOBUF_NONNULL Datum::mutable_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_data();
  // @@protoc_insertion_point(field_mutable:caffe_protos.Datum.data)
  return _s;
}
inline const ::std::string& Datum::_internal_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.data_.Get();
}
inline void Datum::_internal_set_data(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.data_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Datum::_internal_mutable_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.data_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Datum::release_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:caffe_protos.Datum.data)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.data_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.data_.Set("", GetArena());
  }
  return released;
}
inline void Datum::set_allocated_data(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.data_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.data_.IsDefault()) {
    _impl_.data_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:caffe_protos.Datum.data)
}

// optional int32 label = 5;
inline bool Datum::has_label() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  return value;
}
inline void Datum::clear_label() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.label_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline ::int32_t Datum::label() const {
  // @@protoc_insertion_point(field_get:caffe_protos.Datum.label)
  return _internal_label();
}
inline void Datum::set_label(::int32_t value) {
  _internal_set_label(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:caffe_protos.Datum.label)
}
inline ::int32_t Datum::_internal_label() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.label_;
}
inline void Datum::_internal_set_label(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.label_ = value;
}

// repeated float float_data = 6;
inline int Datum::_internal_float_data_size() const {
  return _internal_float_data().size();
}
inline int Datum::float_data_size() const {
  return _internal_float_data_size();
}
inline void Datum::clear_float_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.float_data_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline float Datum::float_data(int index) const {
  // @@protoc_insertion_point(field_get:caffe_protos.Datum.float_data)
  return _internal_float_data().Get(index);
}
inline void Datum::set_float_data(int index, float value) {
  _internal_mutable_float_data()->Set(index, value);
  // @@protoc_insertion_point(field_set:caffe_protos.Datum.float_data)
}
inline void Datum::add_float_data(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_float_data()->InternalAddWithArena(
      internal_visibility(), GetArena(), value);
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:caffe_protos.Datum.float_data)
}
inline const ::google::protobuf::RepeatedField<float>& Datum::float_data() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:caffe_protos.Datum.float_data)
  return _internal_float_data();
}
inline ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL Datum::mutable_float_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:caffe_protos.Datum.float_data)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_float_data();
}
inline const ::google::protobuf::RepeatedField<float>&
Datum::_internal_float_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.float_data_;
}
inline ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL
Datum::_internal_mutable_float_data() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.float_data_;
}

// optional bool encoded = 7 [default = false];
inline bool Datum::has_encoded() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  return value;
}
inline void Datum::clear_encoded() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.encoded_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000080U);
}
inline bool Datum::encoded() const {
  // @@protoc_insertion_point(field_get:caffe_protos.Datum.encoded)
  return _internal_encoded();
}
inline void Datum::set_encoded(bool value) {
  _internal_set_encoded(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:caffe_protos.Datum.encoded)
}
inline bool Datum::_internal_encoded() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.encoded_;
}
inline void Datum::_internal_set_encoded(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.encoded_ = value;
}

// repeated float labels = 8;
inline int Datum::_internal_labels_size() const {
  return _internal_labels().size();
}
inline int Datum::labels_size() const {
  return _internal_labels_size();
}
inline void Datum::clear_labels() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.labels_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000002U);
}
inline float Datum::labels(int index) const {
  // @@protoc_insertion_point(field_get:caffe_protos.Datum.labels)
  return _internal_labels().Get(index);
}
inline void Datum::set_labels(int index, float value) {
  _internal_mutable_labels()->Set(index, value);
  // @@protoc_insertion_point(field_set:caffe_protos.Datum.labels)
}
inline void Datum::add_labels(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _internal_mutable_labels()->InternalAddWithArena(
      internal_visibility(), GetArena(), value);
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:caffe_protos.Datum.labels)
}
inline const ::google::protobuf::RepeatedField<float>& Datum::labels() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:caffe_protos.Datum.labels)
  return _internal_labels();
}
inline ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL Datum::mutable_labels()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:caffe_protos.Datum.labels)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_labels();
}
inline const ::google::protobuf::RepeatedField<float>&
Datum::_internal_labels() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.labels_;
}
inline ::google::protobuf::RepeatedField<float>* PROTOBUF_NONNULL
Datum::_internal_mutable_labels() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.labels_;
}

// -------------------------------------------------------------------

// NormalizedBBox

// optional float xmin = 1;
inline bool NormalizedBBox::has_xmin() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  return value;
}
inline void NormalizedBBox::clear_xmin() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.xmin_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline float NormalizedBBox::xmin() const {
  // @@protoc_insertion_point(field_get:caffe_protos.NormalizedBBox.xmin)
  return _internal_xmin();
}
inline void NormalizedBBox::set_xmin(float value) {
  _internal_set_xmin(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:caffe_protos.NormalizedBBox.xmin)
}
inline float NormalizedBBox::_internal_xmin() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.xmin_;
}
inline void NormalizedBBox::_internal_set_xmin(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.xmin_ = value;
}

// optional float ymin = 2;
inline bool NormalizedBBox::has_ymin() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void NormalizedBBox::clear_ymin() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ymin_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline float NormalizedBBox::ymin() const {
  // @@protoc_insertion_point(field_get:caffe_protos.NormalizedBBox.ymin)
  return _internal_ymin();
}
inline void NormalizedBBox::set_ymin(float value) {
  _internal_set_ymin(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:caffe_protos.NormalizedBBox.ymin)
}
inline float NormalizedBBox::_internal_ymin() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.ymin_;
}
inline void NormalizedBBox::_internal_set_ymin(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ymin_ = value;
}

// optional float xmax = 3;
inline bool NormalizedBBox::has_xmax() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void NormalizedBBox::clear_xmax() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.xmax_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline float NormalizedBBox::xmax() const {
  // @@protoc_insertion_point(field_get:caffe_protos.NormalizedBBox.xmax)
  return _internal_xmax();
}
inline void NormalizedBBox::set_xmax(float value) {
  _internal_set_xmax(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:caffe_protos.NormalizedBBox.xmax)
}
inline float NormalizedBBox::_internal_xmax() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.xmax_;
}
inline void NormalizedBBox::_internal_set_xmax(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.xmax_ = value;
}

// optional float ymax = 4;
inline bool NormalizedBBox::has_ymax() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  return value;
}
inline void NormalizedBBox::clear_ymax() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ymax_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000008U);
}
inline float NormalizedBBox::ymax() const {
  // @@protoc_insertion_point(field_get:caffe_protos.NormalizedBBox.ymax)
  return _internal_ymax();
}
inline void NormalizedBBox::set_ymax(float value) {
  _internal_set_ymax(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:caffe_protos.NormalizedBBox.ymax)
}
inline float NormalizedBBox::_internal_ymax() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.ymax_;
}
inline void NormalizedBBox::_internal_set_ymax(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.ymax_ = value;
}

// optional int32 label = 5;
inline bool NormalizedBBox::has_label() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  return value;
}
inline void NormalizedBBox::clear_label() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.label_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000010U);
}
inline ::int32_t NormalizedBBox::label() const {
  // @@protoc_insertion_point(field_get:caffe_protos.NormalizedBBox.label)
  return _internal_label();
}
inline void NormalizedBBox::set_label(::int32_t value) {
  _internal_set_label(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:caffe_protos.NormalizedBBox.label)
}
inline ::int32_t NormalizedBBox::_internal_label() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.label_;
}
inline void NormalizedBBox::_internal_set_label(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.label_ = value;
}

// optional bool difficult = 6;
inline bool NormalizedBBox::has_difficult() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  return value;
}
inline void NormalizedBBox::clear_difficult() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.difficult_ = false;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000020U);
}
inline bool NormalizedBBox::difficult() const {
  // @@protoc_insertion_point(field_get:caffe_protos.NormalizedBBox.difficult)
  return _internal_difficult();
}
inline void NormalizedBBox::set_difficult(bool value) {
  _internal_set_difficult(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:caffe_protos.NormalizedBBox.difficult)
}
inline bool NormalizedBBox::_internal_difficult() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.difficult_;
}
inline void NormalizedBBox::_internal_set_difficult(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.difficult_ = value;
}

// optional float score = 7;
inline bool NormalizedBBox::has_score() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000040U);
  return value;
}
inline void NormalizedBBox::clear_score() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.score_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000040U);
}
inline float NormalizedBBox::score() const {
  // @@protoc_insertion_point(field_get:caffe_protos.NormalizedBBox.score)
  return _internal_score();
}
inline void NormalizedBBox::set_score(float value) {
  _internal_set_score(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:caffe_protos.NormalizedBBox.score)
}
inline float NormalizedBBox::_internal_score() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.score_;
}
inline void NormalizedBBox::_internal_set_score(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.score_ = value;
}

// optional float size = 8;
inline bool NormalizedBBox::has_size() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000080U);
  return value;
}
inline void NormalizedBBox::clear_size() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000080U);
}
inline float NormalizedBBox::size() const {
  // @@protoc_insertion_point(field_get:caffe_protos.NormalizedBBox.size)
  return _internal_size();
}
inline void NormalizedBBox::set_size(float value) {
  _internal_set_size(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:caffe_protos.NormalizedBBox.size)
}
inline float NormalizedBBox::_internal_size() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.size_;
}
inline void NormalizedBBox::_internal_set_size(float value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.size_ = value;
}

// -------------------------------------------------------------------

// Annotation

// optional int32 instance_id = 1 [default = 0];
inline bool Annotation::has_instance_id() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void Annotation::clear_instance_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.instance_id_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::int32_t Annotation::instance_id() const {
  // @@protoc_insertion_point(field_get:caffe_protos.Annotation.instance_id)
  return _internal_instance_id();
}
inline void Annotation::set_instance_id(::int32_t value) {
  _internal_set_instance_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:caffe_protos.Annotation.instance_id)
}
inline ::int32_t Annotation::_internal_instance_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.instance_id_;
}
inline void Annotation::_internal_set_instance_id(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.instance_id_ = value;
}

// optional .caffe_protos.NormalizedBBox bbox = 2;
inline bool Annotation::has_bbox() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.bbox_ != nullptr);
  return value;
}
inline void Annotation::clear_bbox() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.bbox_ != nullptr) _impl_.bbox_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
inline const ::caffe_protos::NormalizedBBox& Annotation::_internal_bbox() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::caffe_protos::NormalizedBBox* p = _impl_.bbox_;
  return p != nullptr ? *p : reinterpret_cast<const ::caffe_protos::NormalizedBBox&>(::caffe_protos::_NormalizedBBox_default_instance_);
}
inline const ::caffe_protos::NormalizedBBox& Annotation::bbox() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:caffe_protos.Annotation.bbox)
  return _internal_bbox();
}
inline void Annotation::unsafe_arena_set_allocated_bbox(
    ::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.bbox_);
  }
  _impl_.bbox_ = reinterpret_cast<::caffe_protos::NormalizedBBox*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:caffe_protos.Annotation.bbox)
}
inline ::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE Annotation::release_bbox() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::caffe_protos::NormalizedBBox* released = _impl_.bbox_;
  _impl_.bbox_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE Annotation::unsafe_arena_release_bbox() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:caffe_protos.Annotation.bbox)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::caffe_protos::NormalizedBBox* temp = _impl_.bbox_;
  _impl_.bbox_ = nullptr;
  return temp;
}
inline ::caffe_protos::NormalizedBBox* PROTOBUF_NONNULL Annotation::_internal_mutable_bbox() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.bbox_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::caffe_protos::NormalizedBBox>(GetArena());
    _impl_.bbox_ = reinterpret_cast<::caffe_protos::NormalizedBBox*>(p);
  }
  return _impl_.bbox_;
}
inline ::caffe_protos::NormalizedBBox* PROTOBUF_NONNULL Annotation::mutable_bbox()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::caffe_protos::NormalizedBBox* _msg = _internal_mutable_bbox();
  // @@protoc_insertion_point(field_mutable:caffe_protos.Annotation.bbox)
  return _msg;
}
inline void Annotation::set_allocated_bbox(::caffe_protos::NormalizedBBox* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.bbox_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.bbox_ = reinterpret_cast<::caffe_protos::NormalizedBBox*>(value);
  // @@protoc_insertion_point(field_set_allocated:caffe_protos.Annotation.bbox)
}

// -------------------------------------------------------------------

// AnnotationGroup

// optional int32 group_label = 1;
inline bool AnnotationGroup::has_group_label() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  return value;
}
inline void AnnotationGroup::clear_group_label() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.group_label_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline ::int32_t AnnotationGroup::group_label() const {
  // @@protoc_insertion_point(field_get:caffe_protos.AnnotationGroup.group_label)
  return _internal_group_label();
}
inline void AnnotationGroup::set_group_label(::int32_t value) {
  _internal_set_group_label(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:caffe_protos.AnnotationGroup.group_label)
}
inline ::int32_t AnnotationGroup::_internal_group_label() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.group_label_;
}
inline void AnnotationGroup::_internal_set_group_label(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.group_label_ = value;
}

// repeated .caffe_protos.Annotation annotation = 2;
inline int AnnotationGroup::_internal_annotation_size() const {
  return _internal_annotation().size();
}
inline int AnnotationGroup::annotation_size() const {
  return _internal_annotation_size();
}
inline void AnnotationGroup::clear_annotation() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotation_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::caffe_protos::Annotation* PROTOBUF_NONNULL AnnotationGroup::mutable_annotation(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:caffe_protos.AnnotationGroup.annotation)
  return _internal_mutable_annotation()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::caffe_protos::Annotation>* PROTOBUF_NONNULL AnnotationGroup::mutable_annotation()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:caffe_protos.AnnotationGroup.annotation)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_annotation();
}
inline const ::caffe_protos::Annotation& AnnotationGroup::annotation(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:caffe_protos.AnnotationGroup.annotation)
  return _internal_annotation().Get(index);
}
inline ::caffe_protos::Annotation* PROTOBUF_NONNULL AnnotationGroup::add_annotation()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::caffe_protos::Annotation* _add =
      _internal_mutable_annotation()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:caffe_protos.AnnotationGroup.annotation)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::caffe_protos::Annotation>& AnnotationGroup::annotation() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:caffe_protos.AnnotationGroup.annotation)
  return _internal_annotation();
}
inline const ::google::protobuf::RepeatedPtrField<::caffe_protos::Annotation>&
AnnotationGroup::_internal_annotation() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotation_;
}
inline ::google::protobuf::RepeatedPtrField<::caffe_protos::Annotation>* PROTOBUF_NONNULL
AnnotationGroup::_internal_mutable_annotation() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.annotation_;
}

// -------------------------------------------------------------------

// AnnotatedDatum

// optional .caffe_protos.Datum datum = 1;
inline bool AnnotatedDatum::has_datum() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.datum_ != nullptr);
  return value;
}
inline void AnnotatedDatum::clear_datum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.datum_ != nullptr) _impl_.datum_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
inline const ::caffe_protos::Datum& AnnotatedDatum::_internal_datum() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::caffe_protos::Datum* p = _impl_.datum_;
  return p != nullptr ? *p : reinterpret_cast<const ::caffe_protos::Datum&>(::caffe_protos::_Datum_default_instance_);
}
inline const ::caffe_protos::Datum& AnnotatedDatum::datum() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:caffe_protos.AnnotatedDatum.datum)
  return _internal_datum();
}
inline void AnnotatedDatum::unsafe_arena_set_allocated_datum(
    ::caffe_protos::Datum* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.datum_);
  }
  _impl_.datum_ = reinterpret_cast<::caffe_protos::Datum*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:caffe_protos.AnnotatedDatum.datum)
}
inline ::caffe_protos::Datum* PROTOBUF_NULLABLE AnnotatedDatum::release_datum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::caffe_protos::Datum* released = _impl_.datum_;
  _impl_.datum_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::caffe_protos::Datum* PROTOBUF_NULLABLE AnnotatedDatum::unsafe_arena_release_datum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:caffe_protos.AnnotatedDatum.datum)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::caffe_protos::Datum* temp = _impl_.datum_;
  _impl_.datum_ = nullptr;
  return temp;
}
inline ::caffe_protos::Datum* PROTOBUF_NONNULL AnnotatedDatum::_internal_mutable_datum() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.datum_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::caffe_protos::Datum>(GetArena());
    _impl_.datum_ = reinterpret_cast<::caffe_protos::Datum*>(p);
  }
  return _impl_.datum_;
}
inline ::caffe_protos::Datum* PROTOBUF_NONNULL AnnotatedDatum::mutable_datum()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::caffe_protos::Datum* _msg = _internal_mutable_datum();
  // @@protoc_insertion_point(field_mutable:caffe_protos.AnnotatedDatum.datum)
  return _msg;
}
inline void AnnotatedDatum::set_allocated_datum(::caffe_protos::Datum* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.datum_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.datum_ = reinterpret_cast<::caffe_protos::Datum*>(value);
  // @@protoc_insertion_point(field_set_allocated:caffe_protos.AnnotatedDatum.datum)
}

// optional .caffe_protos.AnnotatedDatum.AnnotationType type = 2;
inline bool AnnotatedDatum::has_type() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  return value;
}
inline void AnnotatedDatum::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000004U);
}
inline ::caffe_protos::AnnotatedDatum_AnnotationType AnnotatedDatum::type() const {
  // @@protoc_insertion_point(field_get:caffe_protos.AnnotatedDatum.type)
  return _internal_type();
}
inline void AnnotatedDatum::set_type(::caffe_protos::AnnotatedDatum_AnnotationType value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:caffe_protos.AnnotatedDatum.type)
}
inline ::caffe_protos::AnnotatedDatum_AnnotationType AnnotatedDatum::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return static_cast<::caffe_protos::AnnotatedDatum_AnnotationType>(_impl_.type_);
}
inline void AnnotatedDatum::_internal_set_type(::caffe_protos::AnnotatedDatum_AnnotationType value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);

                                          assert(::google::protobuf::internal::ValidateEnum(
                                              value, ::caffe_protos::AnnotatedDatum_AnnotationType_internal_data_));
                                          _impl_.type_ = value;
}

// repeated .caffe_protos.AnnotationGroup annotation_group = 3;
inline int AnnotatedDatum::_internal_annotation_group_size() const {
  return _internal_annotation_group().size();
}
inline int AnnotatedDatum::annotation_group_size() const {
  return _internal_annotation_group_size();
}
inline void AnnotatedDatum::clear_annotation_group() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.annotation_group_.Clear();
  ClearHasBitForRepeated(_impl_._has_bits_[0],
                  0x00000001U);
}
inline ::caffe_protos::AnnotationGroup* PROTOBUF_NONNULL AnnotatedDatum::mutable_annotation_group(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:caffe_protos.AnnotatedDatum.annotation_group)
  return _internal_mutable_annotation_group()->Mutable(index);
}
inline ::google::protobuf::RepeatedPtrField<::caffe_protos::AnnotationGroup>* PROTOBUF_NONNULL AnnotatedDatum::mutable_annotation_group()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:caffe_protos.AnnotatedDatum.annotation_group)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_annotation_group();
}
inline const ::caffe_protos::AnnotationGroup& AnnotatedDatum::annotation_group(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:caffe_protos.AnnotatedDatum.annotation_group)
  return _internal_annotation_group().Get(index);
}
inline ::caffe_protos::AnnotationGroup* PROTOBUF_NONNULL AnnotatedDatum::add_annotation_group()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::caffe_protos::AnnotationGroup* _add =
      _internal_mutable_annotation_group()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBitForRepeated(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:caffe_protos.AnnotatedDatum.annotation_group)
  return _add;
}
inline const ::google::protobuf::RepeatedPtrField<::caffe_protos::AnnotationGroup>& AnnotatedDatum::annotation_group() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:caffe_protos.AnnotatedDatum.annotation_group)
  return _internal_annotation_group();
}
inline const ::google::protobuf::RepeatedPtrField<::caffe_protos::AnnotationGroup>&
AnnotatedDatum::_internal_annotation_group() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.annotation_group_;
}
inline ::google::protobuf::RepeatedPtrField<::caffe_protos::AnnotationGroup>* PROTOBUF_NONNULL
AnnotatedDatum::_internal_mutable_annotation_group() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.annotation_group_;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace caffe_protos


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::caffe_protos::AnnotatedDatum_AnnotationType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::caffe_protos::AnnotatedDatum_AnnotationType>() {
  return ::caffe_protos::AnnotatedDatum_AnnotationType_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // caffe_5fprotos_2eproto_2epb_2eh
