// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: example.proto
// Protobuf C++ Version: 7.34.1

#ifndef example_2eproto_2epb_2eh
#define example_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7034001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "feature.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_example_2eproto

namespace google {
namespace protobuf {
namespace internal {
template <typename T>
::absl::string_view GetAnyMessageName();
}  // namespace internal
}  // namespace protobuf
}  // namespace google

// Internal implementation detail -- do not use these members.
struct TableStruct_example_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_example_2eproto;
}  // extern "C"
namespace rocal {
namespace tensorflow {
class Example;
struct ExampleDefaultTypeInternal;
extern ExampleDefaultTypeInternal _Example_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull Example_class_data_;
class SequenceExample;
struct SequenceExampleDefaultTypeInternal;
extern SequenceExampleDefaultTypeInternal _SequenceExample_default_instance_;
extern const ::google::protobuf::internal::ClassDataFull SequenceExample_class_data_;
}  // namespace tensorflow
}  // namespace rocal
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace rocal {
namespace tensorflow {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Example final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:rocal.tensorflow.Example) */ {
 public:
  inline Example() : Example(nullptr) {}
  ~Example() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Example* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Example));
  }
#endif

  template <typename = void>
  explicit constexpr Example(::google::protobuf::internal::ConstantInitialized);

  inline Example(const Example& from) : Example(nullptr, from) {}
  inline Example(Example&& from) noexcept
      : Example(nullptr, ::std::move(from)) {}
  inline Example& operator=(const Example& from) {
    CopyFrom(from);
    return *this;
  }
  inline Example& operator=(Example&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Example& default_instance() {
    return *reinterpret_cast<const Example*>(
        &_Example_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Example& a, Example& b) { a.Swap(&b); }
  inline void Swap(Example* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Example* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Example* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<Example>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const Example& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const Example& from) { Example::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Example* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "rocal.tensorflow.Example"; }

  explicit Example(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Example(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Example& from);
  Example(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Example&& from) noexcept
      : Example(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFeaturesFieldNumber = 1,
  };
  // .rocal.tensorflow.Features features = 1;
  [[nodiscard]] bool has_features()
      const;
  void clear_features() ;
  [[nodiscard]] const ::rocal::tensorflow::Features& features() const;
  [[nodiscard]] ::rocal::tensorflow::Features* PROTOBUF_NULLABLE release_features();
  ::rocal::tensorflow::Features* PROTOBUF_NONNULL mutable_features();
  void set_allocated_features(::rocal::tensorflow::Features* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_features(::rocal::tensorflow::Features* PROTOBUF_NULLABLE value);
  ::rocal::tensorflow::Features* PROTOBUF_NULLABLE unsafe_arena_release_features();

  private:
  const ::rocal::tensorflow::Features& _internal_features() const;
  ::rocal::tensorflow::Features* PROTOBUF_NONNULL _internal_mutable_features();

  public:
  // @@protoc_insertion_point(class_scope:rocal.tensorflow.Example)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<0, 1,
                                   1, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Example& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::rocal::tensorflow::Features* PROTOBUF_NULLABLE features_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_example_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull Example_class_data_;
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED SequenceExample final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:rocal.tensorflow.SequenceExample) */ {
 public:
  inline SequenceExample() : SequenceExample(nullptr) {}
  ~SequenceExample() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SequenceExample* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SequenceExample));
  }
#endif

  template <typename = void>
  explicit constexpr SequenceExample(::google::protobuf::internal::ConstantInitialized);

  inline SequenceExample(const SequenceExample& from) : SequenceExample(nullptr, from) {}
  inline SequenceExample(SequenceExample&& from) noexcept
      : SequenceExample(nullptr, ::std::move(from)) {}
  inline SequenceExample& operator=(const SequenceExample& from) {
    CopyFrom(from);
    return *this;
  }
  inline SequenceExample& operator=(SequenceExample&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const SequenceExample& default_instance() {
    return *reinterpret_cast<const SequenceExample*>(
        &_SequenceExample_default_instance_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(SequenceExample& a, SequenceExample& b) { a.Swap(&b); }
  inline void Swap(SequenceExample* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SequenceExample* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] SequenceExample* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return ::google::protobuf::Message::DefaultConstruct<SequenceExample>(arena);
  }
  using ::google::protobuf::Message::CopyFrom;
  void CopyFrom(const SequenceExample& from);
  using ::google::protobuf::Message::MergeFrom;
  void MergeFrom(const SequenceExample& from) { SequenceExample::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SequenceExample* PROTOBUF_NONNULL other);
 private:
  template <typename T>
  friend ::absl::string_view(::google::protobuf::internal::GetAnyMessageName)();
  static ::absl::string_view FullMessageName() { return "rocal.tensorflow.SequenceExample"; }

  explicit SequenceExample(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SequenceExample(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SequenceExample& from);
  SequenceExample(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SequenceExample&& from) noexcept
      : SequenceExample(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_();

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextFieldNumber = 1,
    kFeatureListsFieldNumber = 2,
  };
  // .rocal.tensorflow.Features context = 1;
  [[nodiscard]] bool has_context()
      const;
  void clear_context() ;
  [[nodiscard]] const ::rocal::tensorflow::Features& context() const;
  [[nodiscard]] ::rocal::tensorflow::Features* PROTOBUF_NULLABLE release_context();
  ::rocal::tensorflow::Features* PROTOBUF_NONNULL mutable_context();
  void set_allocated_context(::rocal::tensorflow::Features* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context(::rocal::tensorflow::Features* PROTOBUF_NULLABLE value);
  ::rocal::tensorflow::Features* PROTOBUF_NULLABLE unsafe_arena_release_context();

  private:
  const ::rocal::tensorflow::Features& _internal_context() const;
  ::rocal::tensorflow::Features* PROTOBUF_NONNULL _internal_mutable_context();

  public:
  // .rocal.tensorflow.FeatureLists feature_lists = 2;
  [[nodiscard]] bool has_feature_lists()
      const;
  void clear_feature_lists() ;
  [[nodiscard]] const ::rocal::tensorflow::FeatureLists& feature_lists() const;
  [[nodiscard]] ::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE release_feature_lists();
  ::rocal::tensorflow::FeatureLists* PROTOBUF_NONNULL mutable_feature_lists();
  void set_allocated_feature_lists(::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_feature_lists(::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE value);
  ::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE unsafe_arena_release_feature_lists();

  private:
  const ::rocal::tensorflow::FeatureLists& _internal_feature_lists() const;
  ::rocal::tensorflow::FeatureLists* PROTOBUF_NONNULL _internal_mutable_feature_lists();

  public:
  // @@protoc_insertion_point(class_scope:rocal.tensorflow.SequenceExample)
 private:
  class _Internal;
  friend class ::google::protobuf::internal::TcParser;
  static const ::google::protobuf::internal::TcParseTable<1, 2,
                                   2, 0,
                                   2>
      _table_;

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SequenceExample& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::rocal::tensorflow::Features* PROTOBUF_NULLABLE context_;
    ::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE feature_lists_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_example_2eproto;
};

extern const ::google::protobuf::internal::ClassDataFull SequenceExample_class_data_;

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Example

// .rocal.tensorflow.Features features = 1;
inline bool Example::has_features() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.features_ != nullptr);
  return value;
}
inline const ::rocal::tensorflow::Features& Example::_internal_features() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::rocal::tensorflow::Features* p = _impl_.features_;
  return p != nullptr ? *p : reinterpret_cast<const ::rocal::tensorflow::Features&>(::rocal::tensorflow::_Features_default_instance_);
}
inline const ::rocal::tensorflow::Features& Example::features() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:rocal.tensorflow.Example.features)
  return _internal_features();
}
inline void Example::unsafe_arena_set_allocated_features(
    ::rocal::tensorflow::Features* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.features_);
  }
  _impl_.features_ = reinterpret_cast<::rocal::tensorflow::Features*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:rocal.tensorflow.Example.features)
}
inline ::rocal::tensorflow::Features* PROTOBUF_NULLABLE Example::release_features() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::rocal::tensorflow::Features* released = _impl_.features_;
  _impl_.features_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::rocal::tensorflow::Features* PROTOBUF_NULLABLE Example::unsafe_arena_release_features() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:rocal.tensorflow.Example.features)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::rocal::tensorflow::Features* temp = _impl_.features_;
  _impl_.features_ = nullptr;
  return temp;
}
inline ::rocal::tensorflow::Features* PROTOBUF_NONNULL Example::_internal_mutable_features() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.features_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::rocal::tensorflow::Features>(GetArena());
    _impl_.features_ = reinterpret_cast<::rocal::tensorflow::Features*>(p);
  }
  return _impl_.features_;
}
inline ::rocal::tensorflow::Features* PROTOBUF_NONNULL Example::mutable_features()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::rocal::tensorflow::Features* _msg = _internal_mutable_features();
  // @@protoc_insertion_point(field_mutable:rocal.tensorflow.Example.features)
  return _msg;
}
inline void Example::set_allocated_features(::rocal::tensorflow::Features* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.features_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.features_ = reinterpret_cast<::rocal::tensorflow::Features*>(value);
  // @@protoc_insertion_point(field_set_allocated:rocal.tensorflow.Example.features)
}

// -------------------------------------------------------------------

// SequenceExample

// .rocal.tensorflow.Features context = 1;
inline bool SequenceExample::has_context() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_ != nullptr);
  return value;
}
inline const ::rocal::tensorflow::Features& SequenceExample::_internal_context() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::rocal::tensorflow::Features* p = _impl_.context_;
  return p != nullptr ? *p : reinterpret_cast<const ::rocal::tensorflow::Features&>(::rocal::tensorflow::_Features_default_instance_);
}
inline const ::rocal::tensorflow::Features& SequenceExample::context() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:rocal.tensorflow.SequenceExample.context)
  return _internal_context();
}
inline void SequenceExample::unsafe_arena_set_allocated_context(
    ::rocal::tensorflow::Features* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_);
  }
  _impl_.context_ = reinterpret_cast<::rocal::tensorflow::Features*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:rocal.tensorflow.SequenceExample.context)
}
inline ::rocal::tensorflow::Features* PROTOBUF_NULLABLE SequenceExample::release_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::rocal::tensorflow::Features* released = _impl_.context_;
  _impl_.context_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::rocal::tensorflow::Features* PROTOBUF_NULLABLE SequenceExample::unsafe_arena_release_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:rocal.tensorflow.SequenceExample.context)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::rocal::tensorflow::Features* temp = _impl_.context_;
  _impl_.context_ = nullptr;
  return temp;
}
inline ::rocal::tensorflow::Features* PROTOBUF_NONNULL SequenceExample::_internal_mutable_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::rocal::tensorflow::Features>(GetArena());
    _impl_.context_ = reinterpret_cast<::rocal::tensorflow::Features*>(p);
  }
  return _impl_.context_;
}
inline ::rocal::tensorflow::Features* PROTOBUF_NONNULL SequenceExample::mutable_context()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::rocal::tensorflow::Features* _msg = _internal_mutable_context();
  // @@protoc_insertion_point(field_mutable:rocal.tensorflow.SequenceExample.context)
  return _msg;
}
inline void SequenceExample::set_allocated_context(::rocal::tensorflow::Features* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_ = reinterpret_cast<::rocal::tensorflow::Features*>(value);
  // @@protoc_insertion_point(field_set_allocated:rocal.tensorflow.SequenceExample.context)
}

// .rocal.tensorflow.FeatureLists feature_lists = 2;
inline bool SequenceExample::has_feature_lists() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.feature_lists_ != nullptr);
  return value;
}
inline const ::rocal::tensorflow::FeatureLists& SequenceExample::_internal_feature_lists() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::rocal::tensorflow::FeatureLists* p = _impl_.feature_lists_;
  return p != nullptr ? *p : reinterpret_cast<const ::rocal::tensorflow::FeatureLists&>(::rocal::tensorflow::_FeatureLists_default_instance_);
}
inline const ::rocal::tensorflow::FeatureLists& SequenceExample::feature_lists() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:rocal.tensorflow.SequenceExample.feature_lists)
  return _internal_feature_lists();
}
inline void SequenceExample::unsafe_arena_set_allocated_feature_lists(
    ::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.feature_lists_);
  }
  _impl_.feature_lists_ = reinterpret_cast<::rocal::tensorflow::FeatureLists*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:rocal.tensorflow.SequenceExample.feature_lists)
}
inline ::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE SequenceExample::release_feature_lists() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::rocal::tensorflow::FeatureLists* released = _impl_.feature_lists_;
  _impl_.feature_lists_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE SequenceExample::unsafe_arena_release_feature_lists() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:rocal.tensorflow.SequenceExample.feature_lists)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::rocal::tensorflow::FeatureLists* temp = _impl_.feature_lists_;
  _impl_.feature_lists_ = nullptr;
  return temp;
}
inline ::rocal::tensorflow::FeatureLists* PROTOBUF_NONNULL SequenceExample::_internal_mutable_feature_lists() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.feature_lists_ == nullptr) {
    auto* p = ::google::protobuf::Message::DefaultConstruct<::rocal::tensorflow::FeatureLists>(GetArena());
    _impl_.feature_lists_ = reinterpret_cast<::rocal::tensorflow::FeatureLists*>(p);
  }
  return _impl_.feature_lists_;
}
inline ::rocal::tensorflow::FeatureLists* PROTOBUF_NONNULL SequenceExample::mutable_feature_lists()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::rocal::tensorflow::FeatureLists* _msg = _internal_mutable_feature_lists();
  // @@protoc_insertion_point(field_mutable:rocal.tensorflow.SequenceExample.feature_lists)
  return _msg;
}
inline void SequenceExample::set_allocated_feature_lists(::rocal::tensorflow::FeatureLists* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.feature_lists_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.feature_lists_ = reinterpret_cast<::rocal::tensorflow::FeatureLists*>(value);
  // @@protoc_insertion_point(field_set_allocated:rocal.tensorflow.SequenceExample.feature_lists)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace tensorflow
}  // namespace rocal


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // example_2eproto_2epb_2eh
