// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: example.proto
// Protobuf C++ Version: 7.34.1

#include "example.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace rocal {
namespace tensorflow {

inline constexpr Example::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        features_{nullptr} {}

template <typename>
constexpr Example::Example(::_pbi::ConstantInitialized)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(Example_class_data_.base()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
struct ExampleDefaultTypeInternal {
  constexpr ExampleDefaultTypeInternal() : _instance(::_pbi::ConstantInitialized{}) {}
  ~ExampleDefaultTypeInternal() {}
  union {
    Example _instance;
  };
};

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 ExampleDefaultTypeInternal _Example_default_instance_;

inline constexpr SequenceExample::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        context_{nullptr},
        feature_lists_{nullptr} {}

template <typename>
constexpr SequenceExample::SequenceExample(::_pbi::ConstantInitialized)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(SequenceExample_class_data_.base()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
struct SequenceExampleDefaultTypeInternal {
  constexpr SequenceExampleDefaultTypeInternal() : _instance(::_pbi::ConstantInitialized{}) {}
  ~SequenceExampleDefaultTypeInternal() {}
  union {
    SequenceExample _instance;
  };
};

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 SequenceExampleDefaultTypeInternal _SequenceExample_default_instance_;
}  // namespace tensorflow
}  // namespace rocal
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_example_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_example_2eproto = nullptr;
const ::uint32_t
    TableStruct_example_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::rocal::tensorflow::Example, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::rocal::tensorflow::Example, _impl_.features_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::rocal::tensorflow::SequenceExample, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::rocal::tensorflow::SequenceExample, _impl_.context_),
        PROTOBUF_FIELD_OFFSET(::rocal::tensorflow::SequenceExample, _impl_.feature_lists_),
        0,
        1,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::rocal::tensorflow::Example)},
        {5, sizeof(::rocal::tensorflow::SequenceExample)},
};
static const ::_pb::Message* PROTOBUF_NONNULL const file_default_instances[] = {
    &::rocal::tensorflow::_Example_default_instance_._instance,
    &::rocal::tensorflow::_SequenceExample_default_instance_._instance,
};
const char descriptor_table_protodef_example_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\rexample.proto\022\020rocal.tensorflow\032\rfeatu"
    "re.proto\"7\n\007Example\022,\n\010features\030\001 \001(\0132\032."
    "rocal.tensorflow.Features\"u\n\017SequenceExa"
    "mple\022+\n\007context\030\001 \001(\0132\032.rocal.tensorflow"
    ".Features\0225\n\rfeature_lists\030\002 \001(\0132\036.rocal"
    ".tensorflow.FeatureListsB\201\001\n\026org.tensorf"
    "low.exampleB\rExampleProtosP\001ZSgithub.com"
    "/tensorflow/tensorflow/tensorflow/go/cor"
    "e/example/example_protos_go_proto\370\001\001b\006pr"
    "oto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_example_2eproto_deps[1] = {
        &::descriptor_table_feature_2eproto,
};
static ::absl::once_flag descriptor_table_example_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_example_2eproto = {
    false,
    false,
    364,
    descriptor_table_protodef_example_2eproto,
    "example.proto",
    &descriptor_table_example_2eproto_once,
    descriptor_table_example_2eproto_deps,
    1,
    2,
    schemas,
    file_default_instances,
    TableStruct_example_2eproto::offsets,
    file_level_enum_descriptors_example_2eproto,
    file_level_service_descriptors_example_2eproto,
};
namespace rocal {
namespace tensorflow {
// ===================================================================

class Example::_Internal {
 public:
  using HasBits =
      decltype(::std::declval<Example>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Example, _impl_._has_bits_);
};

void Example::clear_features() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.features_ != nullptr) _impl_.features_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
Example::Example(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Example_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:rocal.tensorflow.Example)
}
PROTOBUF_NDEBUG_INLINE Example::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::rocal::tensorflow::Example& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

Example::Example(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Example& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, Example_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Example* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.features_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.features_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:rocal.tensorflow.Example)
}
PROTOBUF_NDEBUG_INLINE Example::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void Example::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.features_ = {};
}
Example::~Example() {
  // @@protoc_insertion_point(destructor:rocal.tensorflow.Example)
  SharedDtor(*this);
}
inline void Example::SharedDtor(MessageLite& self) {
  Example& this_ = static_cast<Example&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.features_;
  this_._impl_.~Impl_();
}

inline void* PROTOBUF_NONNULL Example::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Example(arena);
}
constexpr auto Example::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Example),
                                            alignof(Example));
}
constexpr auto Example::InternalGenerateClassData_() {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &_Example_default_instance_._instance,
          &_table_.header,
          nullptr,  // IsInitialized
          &Example::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<Example>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Example::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<Example>(), &Example::ByteSizeLong,
              &Example::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Example, _impl_._cached_size_),
          false,
      },
      &Example::kDescriptorMethods,
      &descriptor_table_example_2eproto,
      nullptr,  // tracker
  };
}

PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Example_class_data_ =
        Example::InternalGenerateClassData_();

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Example::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Example_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Example_class_data_.tc_table);
  return Example_class_data_.base();
}
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::_pbi::TcParseTable<0, 1, 1, 0, 2>
Example::_table_ = {
  {
    PROTOBUF_FIELD_OFFSET(Example, _impl_._has_bits_),
    0, // no _extensions_
    1, 0,  // max_field_number, fast_idx_mask
    offsetof(decltype(_table_), field_lookup_table),
    4294967294,  // skipmap
    offsetof(decltype(_table_), field_entries),
    1,  // num_field_entries
    1,  // num_aux_entries
    offsetof(decltype(_table_), aux_entries),
    Example_class_data_.base(),
    nullptr,  // post_loop_handler
    ::_pbi::TcParser::GenericFallback,  // fallback
    #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
    ::_pbi::TcParser::GetTable<::rocal::tensorflow::Example>(),  // to_prefetch
    #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
  }, {{
    // .rocal.tensorflow.Features features = 1;
    {::_pbi::TcParser::FastMtS1,
     {10, 0, 0,
      PROTOBUF_FIELD_OFFSET(Example, _impl_.features_)}},
  }}, {{
    65535, 65535
  }}, {{
    // .rocal.tensorflow.Features features = 1;
    {PROTOBUF_FIELD_OFFSET(Example, _impl_.features_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
  }},
  {{
      {::_pbi::TcParser::GetTable<::rocal::tensorflow::Features>()},
  }},
  {{
  }},
};
PROTOBUF_NOINLINE void Example::Clear() {
// @@protoc_insertion_point(message_clear_start:rocal.tensorflow.Example)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(_impl_.features_ != nullptr);
    _impl_.features_->Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Example::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Example& this_ = static_cast<const Example&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Example::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Example& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:rocal.tensorflow.Example)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .rocal.tensorflow.Features features = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.features_, this_._impl_.features_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:rocal.tensorflow.Example)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Example::ByteSizeLong(const MessageLite& base) {
  const Example& this_ = static_cast<const Example&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Example::ByteSizeLong() const {
  const Example& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:rocal.tensorflow.Example)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

   {
    // .rocal.tensorflow.Features features = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.features_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Example::MergeImpl(::google::protobuf::MessageLite& to_msg,
                            const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this =
      static_cast<Example*>(&to_msg);
  auto& from = static_cast<const Example&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:rocal.tensorflow.Example)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(from._impl_.features_ != nullptr);
    if (_this->_impl_.features_ == nullptr) {
      _this->_impl_.features_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.features_);
    } else {
      _this->_impl_.features_->MergeFrom(*from._impl_.features_);
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Example::CopyFrom(const Example& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:rocal.tensorflow.Example)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Example::InternalSwap(Example* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.features_, other->_impl_.features_);
}

::google::protobuf::Metadata Example::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

class SequenceExample::_Internal {
 public:
  using HasBits =
      decltype(::std::declval<SequenceExample>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_._has_bits_);
};

void SequenceExample::clear_context() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_ != nullptr) _impl_.context_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000001U);
}
void SequenceExample::clear_feature_lists() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.feature_lists_ != nullptr) _impl_.feature_lists_->Clear();
  ClearHasBit(_impl_._has_bits_[0],
                  0x00000002U);
}
SequenceExample::SequenceExample(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SequenceExample_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:rocal.tensorflow.SequenceExample)
}
PROTOBUF_NDEBUG_INLINE SequenceExample::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::rocal::tensorflow::SequenceExample& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0} {}

SequenceExample::SequenceExample(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SequenceExample& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, SequenceExample_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SequenceExample* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.context_)
                : nullptr;
  _impl_.feature_lists_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.feature_lists_)
                : nullptr;

  // @@protoc_insertion_point(copy_constructor:rocal.tensorflow.SequenceExample)
}
PROTOBUF_NDEBUG_INLINE SequenceExample::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void SequenceExample::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_),
           0,
           offsetof(Impl_, feature_lists_) -
               offsetof(Impl_, context_) +
               sizeof(Impl_::feature_lists_));
}
SequenceExample::~SequenceExample() {
  // @@protoc_insertion_point(destructor:rocal.tensorflow.SequenceExample)
  SharedDtor(*this);
}
inline void SequenceExample::SharedDtor(MessageLite& self) {
  SequenceExample& this_ = static_cast<SequenceExample&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_;
  delete this_._impl_.feature_lists_;
  this_._impl_.~Impl_();
}

inline void* PROTOBUF_NONNULL SequenceExample::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SequenceExample(arena);
}
constexpr auto SequenceExample::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(SequenceExample),
                                            alignof(SequenceExample));
}
constexpr auto SequenceExample::InternalGenerateClassData_() {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &_SequenceExample_default_instance_._instance,
          &_table_.header,
          nullptr,  // IsInitialized
          &SequenceExample::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<SequenceExample>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SequenceExample::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<SequenceExample>(), &SequenceExample::ByteSizeLong,
              &SequenceExample::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_._cached_size_),
          false,
      },
      &SequenceExample::kDescriptorMethods,
      &descriptor_table_example_2eproto,
      nullptr,  // tracker
  };
}

PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SequenceExample_class_data_ =
        SequenceExample::InternalGenerateClassData_();

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SequenceExample::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SequenceExample_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SequenceExample_class_data_.tc_table);
  return SequenceExample_class_data_.base();
}
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::_pbi::TcParseTable<1, 2, 2, 0, 2>
SequenceExample::_table_ = {
  {
    PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_._has_bits_),
    0, // no _extensions_
    2, 8,  // max_field_number, fast_idx_mask
    offsetof(decltype(_table_), field_lookup_table),
    4294967292,  // skipmap
    offsetof(decltype(_table_), field_entries),
    2,  // num_field_entries
    2,  // num_aux_entries
    offsetof(decltype(_table_), aux_entries),
    SequenceExample_class_data_.base(),
    nullptr,  // post_loop_handler
    ::_pbi::TcParser::GenericFallback,  // fallback
    #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
    ::_pbi::TcParser::GetTable<::rocal::tensorflow::SequenceExample>(),  // to_prefetch
    #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
  }, {{
    // .rocal.tensorflow.FeatureLists feature_lists = 2;
    {::_pbi::TcParser::FastMtS1,
     {18, 1, 1,
      PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_.feature_lists_)}},
    // .rocal.tensorflow.Features context = 1;
    {::_pbi::TcParser::FastMtS1,
     {10, 0, 0,
      PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_.context_)}},
  }}, {{
    65535, 65535
  }}, {{
    // .rocal.tensorflow.Features context = 1;
    {PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_.context_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    // .rocal.tensorflow.FeatureLists feature_lists = 2;
    {PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_.feature_lists_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
  }},
  {{
      {::_pbi::TcParser::GetTable<::rocal::tensorflow::Features>()},
      {::_pbi::TcParser::GetTable<::rocal::tensorflow::FeatureLists>()},
  }},
  {{
  }},
};
PROTOBUF_NOINLINE void SequenceExample::Clear() {
// @@protoc_insertion_point(message_clear_start:rocal.tensorflow.SequenceExample)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(_impl_.context_ != nullptr);
      _impl_.context_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(_impl_.feature_lists_ != nullptr);
      _impl_.feature_lists_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SequenceExample::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SequenceExample& this_ = static_cast<const SequenceExample&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SequenceExample::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SequenceExample& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:rocal.tensorflow.SequenceExample)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .rocal.tensorflow.Features context = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_, this_._impl_.context_->GetCachedSize(), target,
        stream);
  }

  // .rocal.tensorflow.FeatureLists feature_lists = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.feature_lists_, this_._impl_.feature_lists_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:rocal.tensorflow.SequenceExample)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SequenceExample::ByteSizeLong(const MessageLite& base) {
  const SequenceExample& this_ = static_cast<const SequenceExample&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SequenceExample::ByteSizeLong() const {
  const SequenceExample& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:rocal.tensorflow.SequenceExample)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .rocal.tensorflow.Features context = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_);
    }
    // .rocal.tensorflow.FeatureLists feature_lists = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.feature_lists_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SequenceExample::MergeImpl(::google::protobuf::MessageLite& to_msg,
                            const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this =
      static_cast<SequenceExample*>(&to_msg);
  auto& from = static_cast<const SequenceExample&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:rocal.tensorflow.SequenceExample)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_ != nullptr);
      if (_this->_impl_.context_ == nullptr) {
        _this->_impl_.context_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.context_);
      } else {
        _this->_impl_.context_->MergeFrom(*from._impl_.context_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.feature_lists_ != nullptr);
      if (_this->_impl_.feature_lists_ == nullptr) {
        _this->_impl_.feature_lists_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.feature_lists_);
      } else {
        _this->_impl_.feature_lists_->MergeFrom(*from._impl_.feature_lists_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SequenceExample::CopyFrom(const SequenceExample& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:rocal.tensorflow.SequenceExample)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SequenceExample::InternalSwap(SequenceExample* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_.feature_lists_)
      + sizeof(SequenceExample::_impl_.feature_lists_)
      - PROTOBUF_FIELD_OFFSET(SequenceExample, _impl_.context_)>(
          reinterpret_cast<char*>(&_impl_.context_),
          reinterpret_cast<char*>(&other->_impl_.context_));
}

::google::protobuf::Metadata SequenceExample::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace tensorflow
}  // namespace rocal
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_example_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
