// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: caffe2_protos.proto
// Protobuf C++ Version: 7.34.1

#include "caffe2_protos.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
namespace caffe2_protos {

inline constexpr TensorProto_Segment::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        begin_{::int64_t{0}},
        end_{::int64_t{0}} {}

template <typename>
constexpr TensorProto_Segment::TensorProto_Segment(::_pbi::ConstantInitialized)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(TensorProto_Segment_class_data_.base()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
struct TensorProto_SegmentDefaultTypeInternal {
  constexpr TensorProto_SegmentDefaultTypeInternal() : _instance(::_pbi::ConstantInitialized{}) {}
  ~TensorProto_SegmentDefaultTypeInternal() {}
  union {
    TensorProto_Segment _instance;
  };
};

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 TensorProto_SegmentDefaultTypeInternal _TensorProto_Segment_default_instance_;

inline constexpr TensorProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        dims_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.dims_)>()
        }
        #else
        dims_ {}
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        float_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.float_data_)>()
        }
        #else
        float_data_ {}
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        int32_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.int32_data_)>()
        }
        #else
        int32_data_ {}
        #endif
        ,
        _int32_data_cached_byte_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
        string_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.string_data_)>()
        }
        #else  // !PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
        string_data_ {}
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        double_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.double_data_)>()
        }
        #else
        double_data_ {}
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        int64_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.int64_data_)>()
        }
        #else
        int64_data_ {}
        #endif
        ,
        _int64_data_cached_byte_size_{0},
        byte_data_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        segment_{nullptr},
        data_type_{static_cast< ::caffe2_protos::TensorProto_DataType >(1)} {}

template <typename>
constexpr TensorProto::TensorProto(::_pbi::ConstantInitialized)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(TensorProto_class_data_.base()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
struct TensorProtoDefaultTypeInternal {
  constexpr TensorProtoDefaultTypeInternal() : _instance(::_pbi::ConstantInitialized{}) {}
  ~TensorProtoDefaultTypeInternal() {}
  union {
    TensorProto _instance;
  };
};

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 TensorProtoDefaultTypeInternal _TensorProto_default_instance_;

inline constexpr TensorProtos::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : _cached_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
        protos_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProtos,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProtos, _impl_.protos_)>()
        }
        #else  // !PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
        protos_ {}
        #endif
     {}

template <typename>
constexpr TensorProtos::TensorProtos(::_pbi::ConstantInitialized)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(TensorProtos_class_data_.base()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
struct TensorProtosDefaultTypeInternal {
  constexpr TensorProtosDefaultTypeInternal() : _instance(::_pbi::ConstantInitialized{}) {}
  ~TensorProtosDefaultTypeInternal() {}
  union {
    TensorProtos _instance;
  };
};

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 TensorProtosDefaultTypeInternal _TensorProtos_default_instance_;
}  // namespace caffe2_protos
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_caffe2_5fprotos_2eproto[1];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_caffe2_5fprotos_2eproto = nullptr;
const ::uint32_t
    TableStruct_caffe2_5fprotos_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto_Segment, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto_Segment, _impl_.begin_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto_Segment, _impl_.end_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_._has_bits_),
        13, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.dims_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.data_type_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.float_data_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.int32_data_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.byte_data_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.string_data_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.double_data_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.int64_data_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.name_),
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.segment_),
        0,
        9,
        1,
        2,
        6,
        3,
        4,
        5,
        7,
        8,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProtos, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProtos, _impl_.protos_),
        0,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::caffe2_protos::TensorProto_Segment)},
        {7, sizeof(::caffe2_protos::TensorProto)},
        {30, sizeof(::caffe2_protos::TensorProtos)},
};
static const ::_pb::Message* PROTOBUF_NONNULL const file_default_instances[] = {
    &::caffe2_protos::_TensorProto_Segment_default_instance_._instance,
    &::caffe2_protos::_TensorProto_default_instance_._instance,
    &::caffe2_protos::_TensorProtos_default_instance_._instance,
};
const char descriptor_table_protodef_caffe2_5fprotos_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\023caffe2_protos.proto\022\rcaffe2_protos\"\357\003\n"
    "\013TensorProto\022\014\n\004dims\030\001 \003(\003\022=\n\tdata_type\030"
    "\002 \001(\0162#.caffe2_protos.TensorProto.DataTy"
    "pe:\005FLOAT\022\026\n\nfloat_data\030\003 \003(\002B\002\020\001\022\026\n\nint"
    "32_data\030\004 \003(\005B\002\020\001\022\021\n\tbyte_data\030\005 \001(\014\022\023\n\013"
    "string_data\030\006 \003(\014\022\027\n\013double_data\030\t \003(\001B\002"
    "\020\001\022\026\n\nint64_data\030\n \003(\003B\002\020\001\022\014\n\004name\030\007 \001(\t"
    "\0223\n\007segment\030\013 \001(\0132\".caffe2_protos.Tensor"
    "Proto.Segment\032%\n\007Segment\022\r\n\005begin\030\001 \002(\003\022"
    "\013\n\003end\030\002 \002(\003\"\237\001\n\010DataType\022\r\n\tUNDEFINED\020\000"
    "\022\t\n\005FLOAT\020\001\022\t\n\005INT32\020\002\022\010\n\004BYTE\020\003\022\n\n\006STRI"
    "NG\020\004\022\010\n\004BOOL\020\005\022\t\n\005UINT8\020\006\022\010\n\004INT8\020\007\022\n\n\006U"
    "INT16\020\010\022\t\n\005INT16\020\t\022\t\n\005INT64\020\n\022\013\n\007FLOAT16"
    "\020\014\022\n\n\006DOUBLE\020\r\":\n\014TensorProtos\022*\n\006protos"
    "\030\001 \003(\0132\032.caffe2_protos.TensorProto"
};
static ::absl::once_flag descriptor_table_caffe2_5fprotos_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_caffe2_5fprotos_2eproto = {
    false,
    false,
    594,
    descriptor_table_protodef_caffe2_5fprotos_2eproto,
    "caffe2_protos.proto",
    &descriptor_table_caffe2_5fprotos_2eproto_once,
    nullptr,
    0,
    3,
    schemas,
    file_default_instances,
    TableStruct_caffe2_5fprotos_2eproto::offsets,
    file_level_enum_descriptors_caffe2_5fprotos_2eproto,
    file_level_service_descriptors_caffe2_5fprotos_2eproto,
};
namespace caffe2_protos {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
TensorProto_DataType_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_caffe2_5fprotos_2eproto);
  return file_level_enum_descriptors_caffe2_5fprotos_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t TensorProto_DataType_internal_data_[] = {
    720896u, 32u, 6u, };
// ===================================================================

class TensorProto_Segment::_Internal {
 public:
  using HasBits =
      decltype(::std::declval<TensorProto_Segment>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_._has_bits_);
  static bool MissingRequiredFields(const HasBits& has_bits) {
    return ((has_bits[0] & 0x00000003) ^ 0x00000003) != 0;
  }
};

TensorProto_Segment::TensorProto_Segment(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TensorProto_Segment_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:caffe2_protos.TensorProto.Segment)
}
TensorProto_Segment::TensorProto_Segment(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TensorProto_Segment& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TensorProto_Segment_class_data_.base()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE TensorProto_Segment::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0} {}

inline void TensorProto_Segment::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, begin_),
           0,
           offsetof(Impl_, end_) -
               offsetof(Impl_, begin_) +
               sizeof(Impl_::end_));
}
TensorProto_Segment::~TensorProto_Segment() {
  // @@protoc_insertion_point(destructor:caffe2_protos.TensorProto.Segment)
  SharedDtor(*this);
}
inline void TensorProto_Segment::SharedDtor(MessageLite& self) {
  TensorProto_Segment& this_ = static_cast<TensorProto_Segment&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

inline void* PROTOBUF_NONNULL TensorProto_Segment::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TensorProto_Segment(arena);
}
constexpr auto TensorProto_Segment::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(TensorProto_Segment),
                                            alignof(TensorProto_Segment));
}
constexpr auto TensorProto_Segment::InternalGenerateClassData_() {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &_TensorProto_Segment_default_instance_._instance,
          &_table_.header,
          TensorProto_Segment::IsInitializedImpl,
          &TensorProto_Segment::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<TensorProto_Segment>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TensorProto_Segment::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<TensorProto_Segment>(), &TensorProto_Segment::ByteSizeLong,
              &TensorProto_Segment::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_._cached_size_),
          false,
      },
      &TensorProto_Segment::kDescriptorMethods,
      &descriptor_table_caffe2_5fprotos_2eproto,
      nullptr,  // tracker
  };
}

PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TensorProto_Segment_class_data_ =
        TensorProto_Segment::InternalGenerateClassData_();

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TensorProto_Segment::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TensorProto_Segment_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TensorProto_Segment_class_data_.tc_table);
  return TensorProto_Segment_class_data_.base();
}
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::_pbi::TcParseTable<1, 2, 0, 0, 2>
TensorProto_Segment::_table_ = {
  {
    PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_._has_bits_),
    0, // no _extensions_
    2, 8,  // max_field_number, fast_idx_mask
    offsetof(decltype(_table_), field_lookup_table),
    4294967292,  // skipmap
    offsetof(decltype(_table_), field_entries),
    2,  // num_field_entries
    0,  // num_aux_entries
    offsetof(decltype(_table_), field_names),  // no aux_entries
    TensorProto_Segment_class_data_.base(),
    nullptr,  // post_loop_handler
    ::_pbi::TcParser::GenericFallback,  // fallback
    #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
    ::_pbi::TcParser::GetTable<::caffe2_protos::TensorProto_Segment>(),  // to_prefetch
    #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
  }, {{
    // required int64 end = 2;
    {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(TensorProto_Segment, _impl_.end_), 1>(),
     {16, 1, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_.end_)}},
    // required int64 begin = 1;
    {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(TensorProto_Segment, _impl_.begin_), 0>(),
     {8, 0, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_.begin_)}},
  }}, {{
    65535, 65535
  }}, {{
    // required int64 begin = 1;
    {PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_.begin_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    // required int64 end = 2;
    {PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_.end_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
  }},
  // no aux_entries
  {{
  }},
};
PROTOBUF_NOINLINE void TensorProto_Segment::Clear() {
// @@protoc_insertion_point(message_clear_start:caffe2_protos.TensorProto.Segment)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&_impl_.begin_, 0, static_cast<::size_t>(
        reinterpret_cast<char*>(&_impl_.end_) -
        reinterpret_cast<char*>(&_impl_.begin_)) + sizeof(_impl_.end_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TensorProto_Segment::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TensorProto_Segment& this_ = static_cast<const TensorProto_Segment&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TensorProto_Segment::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TensorProto_Segment& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:caffe2_protos.TensorProto.Segment)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // required int64 begin = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
            stream, this_._internal_begin(), target);
  }

  // required int64 end = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target =
        ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
            stream, this_._internal_end(), target);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:caffe2_protos.TensorProto.Segment)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TensorProto_Segment::ByteSizeLong(const MessageLite& base) {
  const TensorProto_Segment& this_ = static_cast<const TensorProto_Segment&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TensorProto_Segment::ByteSizeLong() const {
  const TensorProto_Segment& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:caffe2_protos.TensorProto.Segment)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // required int64 begin = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
          this_._internal_begin());
    }
    // required int64 end = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
          this_._internal_end());
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TensorProto_Segment::MergeImpl(::google::protobuf::MessageLite& to_msg,
                            const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this =
      static_cast<TensorProto_Segment*>(&to_msg);
  auto& from = static_cast<const TensorProto_Segment&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:caffe2_protos.TensorProto.Segment)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_impl_.begin_ = from._impl_.begin_;
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_impl_.end_ = from._impl_.end_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TensorProto_Segment::CopyFrom(const TensorProto_Segment& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:caffe2_protos.TensorProto.Segment)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

PROTOBUF_NOINLINE bool TensorProto_Segment::IsInitializedImpl(
    const MessageLite& msg) {
  auto& this_ = static_cast<const TensorProto_Segment&>(msg);
  if (_Internal::MissingRequiredFields(this_._impl_._has_bits_)) {
    return false;
  }
  return true;
}

void TensorProto_Segment::InternalSwap(TensorProto_Segment* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_.end_)
      + sizeof(TensorProto_Segment::_impl_.end_)
      - PROTOBUF_FIELD_OFFSET(TensorProto_Segment, _impl_.begin_)>(
          reinterpret_cast<char*>(&_impl_.begin_),
          reinterpret_cast<char*>(&other->_impl_.begin_));
}

::google::protobuf::Metadata TensorProto_Segment::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

class TensorProto::_Internal {
 public:
  using HasBits =
      decltype(::std::declval<TensorProto>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TensorProto, _impl_._has_bits_);
};

TensorProto::TensorProto(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TensorProto_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:caffe2_protos.TensorProto)
}
PROTOBUF_NDEBUG_INLINE TensorProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::caffe2_protos::TensorProto& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        dims_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.dims_)>()
        , from.dims_}
        #else
        dims_ { visibility, arena, from.dims_ }
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        float_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.float_data_)>()
        , from.float_data_}
        #else
        float_data_ { visibility, arena, from.float_data_ }
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        int32_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.int32_data_)>()
        , from.int32_data_}
        #else
        int32_data_ { visibility, arena, from.int32_data_ }
        #endif
        ,
        _int32_data_cached_byte_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
        string_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.string_data_)>()
        , from.string_data_}
        #else
        string_data_ { visibility, arena, from.string_data_ }
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        double_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.double_data_)>()
        , from.double_data_}
        #else
        double_data_ { visibility, arena, from.double_data_ }
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        int64_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.int64_data_)>()
        , from.int64_data_}
        #else
        int64_data_ { visibility, arena, from.int64_data_ }
        #endif
        ,
        _int64_data_cached_byte_size_{0},
        byte_data_(arena, from.byte_data_),
        name_(arena, from.name_) {}

TensorProto::TensorProto(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TensorProto& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TensorProto_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TensorProto* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.segment_ = (CheckHasBit(cached_has_bits, 0x00000100U))
                ? ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.segment_)
                : nullptr;
  _impl_.data_type_ = from._impl_.data_type_;

  // @@protoc_insertion_point(copy_constructor:caffe2_protos.TensorProto)
}
PROTOBUF_NDEBUG_INLINE TensorProto::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        dims_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.dims_)>()
        }
        #else
        dims_ { visibility, arena }
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        float_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.float_data_)>()
        }
        #else
        float_data_ { visibility, arena }
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        int32_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.int32_data_)>()
        }
        #else
        int32_data_ { visibility, arena }
        #endif
        ,
        _int32_data_cached_byte_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
        string_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.string_data_)>()
        }
        #else
        string_data_ { visibility, arena }
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        double_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.double_data_)>()
        }
        #else
        double_data_ { visibility, arena }
        #endif
        ,
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_FIELD
        int64_data_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProto,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProto, _impl_.int64_data_)>()
        }
        #else
        int64_data_ { visibility, arena }
        #endif
        ,
        _int64_data_cached_byte_size_{0},
        byte_data_(arena),
        name_(arena),
        data_type_{static_cast< ::caffe2_protos::TensorProto_DataType >(1)} {}

inline void TensorProto::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.segment_ = {};
}
TensorProto::~TensorProto() {
  // @@protoc_insertion_point(destructor:caffe2_protos.TensorProto)
  SharedDtor(*this);
}
inline void TensorProto::SharedDtor(MessageLite& self) {
  TensorProto& this_ = static_cast<TensorProto&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.byte_data_.Destroy();
  this_._impl_.name_.Destroy();
  delete this_._impl_.segment_;
  this_._impl_.~Impl_();
}

inline void* PROTOBUF_NONNULL TensorProto::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TensorProto(arena);
}
#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
constexpr auto TensorProto::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(TensorProto),
                                            alignof(TensorProto));
}
#else  // !PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
constexpr auto TensorProto::InternalNewImpl_() {
  constexpr auto arena_bits = ::google::protobuf::internal::EncodePlacementArenaOffsets({
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.dims_) +
          decltype(TensorProto::_impl_.dims_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.float_data_) +
          decltype(TensorProto::_impl_.float_data_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int32_data_) +
          decltype(TensorProto::_impl_.int32_data_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.string_data_) +
          decltype(TensorProto::_impl_.string_data_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.double_data_) +
          decltype(TensorProto::_impl_.double_data_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int64_data_) +
          decltype(TensorProto::_impl_.int64_data_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
  });
  if (arena_bits.has_value()) {
    return ::google::protobuf::internal::MessageCreator::CopyInit(
        sizeof(TensorProto), alignof(TensorProto), *arena_bits);
  } else {
    return ::google::protobuf::internal::MessageCreator(&TensorProto::PlacementNew_,
                                 sizeof(TensorProto),
                                 alignof(TensorProto));
  }
}
#endif
constexpr auto TensorProto::InternalGenerateClassData_() {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &_TensorProto_default_instance_._instance,
          &_table_.header,
          TensorProto::IsInitializedImpl,
          &TensorProto::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<TensorProto>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TensorProto::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<TensorProto>(), &TensorProto::ByteSizeLong,
              &TensorProto::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TensorProto, _impl_._cached_size_),
          false,
      },
      &TensorProto::kDescriptorMethods,
      &descriptor_table_caffe2_5fprotos_2eproto,
      nullptr,  // tracker
  };
}

PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TensorProto_class_data_ =
        TensorProto::InternalGenerateClassData_();

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TensorProto::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TensorProto_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TensorProto_class_data_.tc_table);
  return TensorProto_class_data_.base();
}
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::_pbi::TcParseTable<4, 10, 2, 0, 2>
TensorProto::_table_ = {
  {
    PROTOBUF_FIELD_OFFSET(TensorProto, _impl_._has_bits_),
    0, // no _extensions_
    11, 120,  // max_field_number, fast_idx_mask
    offsetof(decltype(_table_), field_lookup_table),
    4294965376,  // skipmap
    offsetof(decltype(_table_), field_entries),
    10,  // num_field_entries
    2,  // num_aux_entries
    offsetof(decltype(_table_), aux_entries),
    TensorProto_class_data_.base(),
    nullptr,  // post_loop_handler
    ::_pbi::TcParser::GenericFallback,  // fallback
    #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
    ::_pbi::TcParser::GetTable<::caffe2_protos::TensorProto>(),  // to_prefetch
    #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
  }, {{
    {::_pbi::TcParser::MiniParse, {}},
    // repeated int64 dims = 1;
    {::_pbi::TcParser::FastV64R1,
     {8, 0, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.dims_)}},
    // optional .caffe2_protos.TensorProto.DataType data_type = 2 [default = FLOAT];
    {::_pbi::TcParser::FastEvS1,
     {16, 9, 1,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.data_type_)}},
    // repeated float float_data = 3 [packed = true];
    {::_pbi::TcParser::FastF32P1,
     {26, 1, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.float_data_)}},
    // repeated int32 int32_data = 4 [packed = true];
    {::_pbi::TcParser::FastV32P1,
     {34, 2, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int32_data_)}},
    // optional bytes byte_data = 5;
    {::_pbi::TcParser::FastBS1,
     {42, 6, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.byte_data_)}},
    // repeated bytes string_data = 6;
    {::_pbi::TcParser::FastBR1,
     {50, 3, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.string_data_)}},
    // optional string name = 7;
    {::_pbi::TcParser::FastBS1,
     {58, 7, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.name_)}},
    {::_pbi::TcParser::MiniParse, {}},
    // repeated double double_data = 9 [packed = true];
    {::_pbi::TcParser::FastF64P1,
     {74, 4, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.double_data_)}},
    // repeated int64 int64_data = 10 [packed = true];
    {::_pbi::TcParser::FastV64P1,
     {82, 5, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int64_data_)}},
    // optional .caffe2_protos.TensorProto.Segment segment = 11;
    {::_pbi::TcParser::FastMtS1,
     {90, 8, 0,
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.segment_)}},
    {::_pbi::TcParser::MiniParse, {}},
    {::_pbi::TcParser::MiniParse, {}},
    {::_pbi::TcParser::MiniParse, {}},
    {::_pbi::TcParser::MiniParse, {}},
  }}, {{
    65535, 65535
  }}, {{
    // repeated int64 dims = 1;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.dims_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kInt64)},
    // optional .caffe2_protos.TensorProto.DataType data_type = 2 [default = FLOAT];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.data_type_), _Internal::kHasBitsOffset + 9, 1, (0 | ::_fl::kFcOptional | ::_fl::kEnum)},
    // repeated float float_data = 3 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.float_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedFloat)},
    // repeated int32 int32_data = 4 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int32_data_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt32)},
    // optional bytes byte_data = 5;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.byte_data_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    // repeated bytes string_data = 6;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.string_data_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcRepeated | ::_fl::kBytes | ::_fl::kRepSString)},
    // optional string name = 7;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.name_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    // repeated double double_data = 9 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.double_data_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedDouble)},
    // repeated int64 int64_data = 10 [packed = true];
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.int64_data_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcRepeated | ::_fl::kPackedInt64)},
    // optional .caffe2_protos.TensorProto.Segment segment = 11;
    {PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.segment_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
  }},
  {{
      {::_pbi::TcParser::GetTable<::caffe2_protos::TensorProto_Segment>()},
      {::_pbi::FieldAuxEnumData{}, ::caffe2_protos::TensorProto_DataType_internal_data_},
  }},
  {{
  }},
};
PROTOBUF_NOINLINE void TensorProto::Clear() {
// @@protoc_insertion_point(message_clear_start:caffe2_protos.TensorProto)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000001U)) {
      _impl_.dims_.Clear();
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000002U)) {
      _impl_.float_data_.Clear();
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000004U)) {
      _impl_.int32_data_.Clear();
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000008U)) {
      _impl_.string_data_.Clear();
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000010U)) {
      _impl_.double_data_.Clear();
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000020U)) {
      _impl_.int64_data_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _impl_.byte_data_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      _impl_.name_.ClearNonDefaultToEmpty();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000300U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(_impl_.segment_ != nullptr);
      _impl_.segment_->Clear();
    }
    _impl_.data_type_ = 1;
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TensorProto::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TensorProto& this_ = static_cast<const TensorProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TensorProto::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TensorProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:caffe2_protos.TensorProto)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated int64 dims = 1;
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_dims_size(); i < n; ++i) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteInt64ToArray(
          1, this_._internal_dims().Get(i), target);
    }
  }

  // optional .caffe2_protos.TensorProto.DataType data_type = 2 [default = FLOAT];
  if (CheckHasBit(cached_has_bits, 0x00000200U)) {
    target = stream->EnsureSpace(target);
    target = ::_pbi::WireFormatLite::WriteEnumToArray(
        2, this_._internal_data_type(), target);
  }

  // repeated float float_data = 3 [packed = true];
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000002U)) {
    if (this_._internal_float_data_size() > 0) {
      target = stream->WriteFixedPacked(3, this_._internal_float_data(), target);
    }
  }

  // repeated int32 int32_data = 4 [packed = true];
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000004U)) {
    {
      int byte_size = this_._impl_._int32_data_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteInt32Packed(
            4, this_._internal_int32_data(), byte_size, target);
      }
    }
  }

  // optional bytes byte_data = 5;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    const ::std::string& _s = this_._internal_byte_data();
    target = stream->WriteBytesMaybeAliased(5, _s, target);
  }

  // repeated bytes string_data = 6;
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000008U)) {
    for (int i = 0, n = this_._internal_string_data_size(); i < n; ++i) {
      const auto& s = this_._internal_string_data().Get(i);
      target = stream->WriteBytes(6, s, target);
    }
  }

  // optional string name = 7;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    const ::std::string& _s = this_._internal_name();
    target = stream->WriteStringMaybeAliased(7, _s, target);
  }

  // repeated double double_data = 9 [packed = true];
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000010U)) {
    if (this_._internal_double_data_size() > 0) {
      target = stream->WriteFixedPacked(9, this_._internal_double_data(), target);
    }
  }

  // repeated int64 int64_data = 10 [packed = true];
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000020U)) {
    {
      int byte_size = this_._impl_._int64_data_cached_byte_size_.Get();
      if (byte_size > 0) {
        target = stream->WriteInt64Packed(
            10, this_._internal_int64_data(), byte_size, target);
      }
    }
  }

  // optional .caffe2_protos.TensorProto.Segment segment = 11;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        11, *this_._impl_.segment_, this_._impl_.segment_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:caffe2_protos.TensorProto)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TensorProto::ByteSizeLong(const MessageLite& base) {
  const TensorProto& this_ = static_cast<const TensorProto&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TensorProto::ByteSizeLong() const {
  const TensorProto& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:caffe2_protos.TensorProto)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated int64 dims = 1;
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000001U)) {
      ::size_t data_size = ::_pbi::WireFormatLite::Int64Size(
          this_._internal_dims());
      ::size_t tag_size = ::size_t{1} *
          ::_pbi::FromIntSize(this_._internal_dims_size());
      total_size += tag_size + data_size;
    }
    // repeated float float_data = 3 [packed = true];
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000002U)) {
      ::size_t data_size = ::size_t{4} *
          ::_pbi::FromIntSize(this_._internal_float_data_size());
      ::size_t tag_size = data_size == 0
          ? 0
          : 1 + ::_pbi::WireFormatLite::Int32Size(
                              static_cast<::int32_t>(data_size));
      total_size += tag_size + data_size;
    }
    // repeated int32 int32_data = 4 [packed = true];
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000004U)) {
      total_size +=
          ::_pbi::WireFormatLite::Int32SizeWithPackedTagSize(
              this_._internal_int32_data(), 1,
              this_._impl_._int32_data_cached_byte_size_);
    }
    // repeated bytes string_data = 6;
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000008U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_string_data().size());
      for (int i = 0, n = this_._internal_string_data().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::BytesSize(
            this_._internal_string_data().Get(i));
      }
    }
    // repeated double double_data = 9 [packed = true];
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000010U)) {
      ::size_t data_size = ::size_t{8} *
          ::_pbi::FromIntSize(this_._internal_double_data_size());
      ::size_t tag_size = data_size == 0
          ? 0
          : 1 + ::_pbi::WireFormatLite::Int32Size(
                              static_cast<::int32_t>(data_size));
      total_size += tag_size + data_size;
    }
    // repeated int64 int64_data = 10 [packed = true];
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000020U)) {
      total_size +=
          ::_pbi::WireFormatLite::Int64SizeWithPackedTagSize(
              this_._internal_int64_data(), 1,
              this_._impl_._int64_data_cached_byte_size_);
    }
    // optional bytes byte_data = 5;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                      this_._internal_byte_data());
    }
    // optional string name = 7;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                      this_._internal_name());
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000300U)) {
    // optional .caffe2_protos.TensorProto.Segment segment = 11;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.segment_);
    }
    // optional .caffe2_protos.TensorProto.DataType data_type = 2 [default = FLOAT];
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      total_size += 1 +
                    ::_pbi::WireFormatLite::EnumSize(this_._internal_data_type());
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TensorProto::MergeImpl(::google::protobuf::MessageLite& to_msg,
                            const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this =
      static_cast<TensorProto*>(&to_msg);
  auto& from = static_cast<const TensorProto&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:caffe2_protos.TensorProto)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_dims()->MergeFrom(from._internal_dims());
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_float_data()->MergeFrom(from._internal_float_data());
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000004U)) {
      _this->_internal_mutable_int32_data()->MergeFrom(from._internal_int32_data());
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000008U)) {
      _this->_internal_mutable_string_data()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_string_data());
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000010U)) {
      _this->_internal_mutable_double_data()->MergeFrom(from._internal_double_data());
    }
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000020U)) {
      _this->_internal_mutable_int64_data()->MergeFrom(from._internal_int64_data());
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      _this->_internal_set_byte_data(from._internal_byte_data());
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      _this->_internal_set_name(from._internal_name());
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000300U)) {
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      ABSL_DCHECK(from._impl_.segment_ != nullptr);
      if (_this->_impl_.segment_ == nullptr) {
        _this->_impl_.segment_ = ::google::protobuf::Message::CopyConstruct(arena, *from._impl_.segment_);
      } else {
        _this->_impl_.segment_->MergeFrom(*from._impl_.segment_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000200U)) {
      _this->_impl_.data_type_ = from._impl_.data_type_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TensorProto::CopyFrom(const TensorProto& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:caffe2_protos.TensorProto)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

PROTOBUF_NOINLINE bool TensorProto::IsInitializedImpl(
    const MessageLite& msg) {
  auto& this_ = static_cast<const TensorProto&>(msg);
  if (CheckHasBit(this_._impl_._has_bits_[0], 0x00000100U)) {
    if (!this_._impl_.segment_->IsInitialized()) return false;
  }
  return true;
}

void TensorProto::InternalSwap(TensorProto* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.dims_.InternalSwap(&other->_impl_.dims_);
  _impl_.float_data_.InternalSwap(&other->_impl_.float_data_);
  _impl_.int32_data_.InternalSwap(&other->_impl_.int32_data_);
  _impl_.string_data_.InternalSwap(&other->_impl_.string_data_);
  _impl_.double_data_.InternalSwap(&other->_impl_.double_data_);
  _impl_.int64_data_.InternalSwap(&other->_impl_.int64_data_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.byte_data_, &other->_impl_.byte_data_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.name_, &other->_impl_.name_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.data_type_)
      + sizeof(TensorProto::_impl_.data_type_)
      - PROTOBUF_FIELD_OFFSET(TensorProto, _impl_.segment_)>(
          reinterpret_cast<char*>(&_impl_.segment_),
          reinterpret_cast<char*>(&other->_impl_.segment_));
}

::google::protobuf::Metadata TensorProto::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

class TensorProtos::_Internal {
 public:
  using HasBits =
      decltype(::std::declval<TensorProtos>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TensorProtos, _impl_._has_bits_);
};

TensorProtos::TensorProtos(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TensorProtos_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:caffe2_protos.TensorProtos)
}
PROTOBUF_NDEBUG_INLINE TensorProtos::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::caffe2_protos::TensorProtos& from_msg)
      : _has_bits_{from._has_bits_},
        _cached_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
        protos_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProtos,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProtos, _impl_.protos_)>()
        , from.protos_}
        #else
        protos_ { visibility, arena, from.protos_ }
        #endif
     {}

TensorProtos::TensorProtos(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TensorProtos& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : ::google::protobuf::Message(arena, TensorProtos_class_data_.base()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : ::google::protobuf::Message(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TensorProtos* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);

  // @@protoc_insertion_point(copy_constructor:caffe2_protos.TensorProtos)
}
PROTOBUF_NDEBUG_INLINE TensorProtos::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : _cached_size_{0},
        #ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
        protos_{visibility, ::_pbi::InternalMetadataOffset::Build<
            ::caffe2_protos::TensorProtos,
            PROTOBUF_FIELD_OFFSET(::caffe2_protos::TensorProtos, _impl_.protos_)>()
        }
        #else
        protos_ { visibility, arena }
        #endif
     {}

inline void TensorProtos::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
}
TensorProtos::~TensorProtos() {
  // @@protoc_insertion_point(destructor:caffe2_protos.TensorProtos)
  SharedDtor(*this);
}
inline void TensorProtos::SharedDtor(MessageLite& self) {
  TensorProtos& this_ = static_cast<TensorProtos&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

inline void* PROTOBUF_NONNULL TensorProtos::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TensorProtos(arena);
}
#ifdef PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
constexpr auto TensorProtos::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(TensorProtos),
                                            alignof(TensorProtos));
}
#else  // !PROTOBUF_INTERNAL_REMOVE_ARENA_PTRS_REPEATED_PTR_FIELD
constexpr auto TensorProtos::InternalNewImpl_() {
  constexpr auto arena_bits = ::google::protobuf::internal::EncodePlacementArenaOffsets({
      PROTOBUF_FIELD_OFFSET(TensorProtos, _impl_.protos_) +
          decltype(TensorProtos::_impl_.protos_)::
              InternalGetArenaOffset(
                  ::google::protobuf::Message::internal_visibility()),
  });
  if (arena_bits.has_value()) {
    return ::google::protobuf::internal::MessageCreator::ZeroInit(
        sizeof(TensorProtos), alignof(TensorProtos), *arena_bits);
  } else {
    return ::google::protobuf::internal::MessageCreator(&TensorProtos::PlacementNew_,
                                 sizeof(TensorProtos),
                                 alignof(TensorProtos));
  }
}
#endif
constexpr auto TensorProtos::InternalGenerateClassData_() {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &_TensorProtos_default_instance_._instance,
          &_table_.header,
          TensorProtos::IsInitializedImpl,
          &TensorProtos::MergeImpl,
          ::google::protobuf::Message::GetNewImpl<TensorProtos>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TensorProtos::SharedDtor,
          ::google::protobuf::Message::GetClearImpl<TensorProtos>(), &TensorProtos::ByteSizeLong,
              &TensorProtos::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TensorProtos, _impl_._cached_size_),
          false,
      },
      &TensorProtos::kDescriptorMethods,
      &descriptor_table_caffe2_5fprotos_2eproto,
      nullptr,  // tracker
  };
}

PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TensorProtos_class_data_ =
        TensorProtos::InternalGenerateClassData_();

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TensorProtos::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TensorProtos_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TensorProtos_class_data_.tc_table);
  return TensorProtos_class_data_.base();
}
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1
const ::_pbi::TcParseTable<0, 1, 1, 0, 2>
TensorProtos::_table_ = {
  {
    PROTOBUF_FIELD_OFFSET(TensorProtos, _impl_._has_bits_),
    0, // no _extensions_
    1, 0,  // max_field_number, fast_idx_mask
    offsetof(decltype(_table_), field_lookup_table),
    4294967294,  // skipmap
    offsetof(decltype(_table_), field_entries),
    1,  // num_field_entries
    1,  // num_aux_entries
    offsetof(decltype(_table_), aux_entries),
    TensorProtos_class_data_.base(),
    nullptr,  // post_loop_handler
    ::_pbi::TcParser::GenericFallback,  // fallback
    #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
    ::_pbi::TcParser::GetTable<::caffe2_protos::TensorProtos>(),  // to_prefetch
    #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
  }, {{
    // repeated .caffe2_protos.TensorProto protos = 1;
    {::_pbi::TcParser::FastMtR1,
     {10, 0, 0,
      PROTOBUF_FIELD_OFFSET(TensorProtos, _impl_.protos_)}},
  }}, {{
    65535, 65535
  }}, {{
    // repeated .caffe2_protos.TensorProto protos = 1;
    {PROTOBUF_FIELD_OFFSET(TensorProtos, _impl_.protos_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kMessage | ::_fl::kTvTable)},
  }},
  {{
      {::_pbi::TcParser::GetTable<::caffe2_protos::TensorProto>()},
  }},
  {{
  }},
};
PROTOBUF_NOINLINE void TensorProtos::Clear() {
// @@protoc_insertion_point(message_clear_start:caffe2_protos.TensorProtos)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void) cached_has_bits;

  cached_has_bits = _impl_._has_bits_[0];
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000001U)) {
    _impl_.protos_.Clear();
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TensorProtos::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TensorProtos& this_ = static_cast<const TensorProtos&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TensorProtos::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TensorProtos& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:caffe2_protos.TensorProtos)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // repeated .caffe2_protos.TensorProto protos = 1;
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000001U)) {
    for (unsigned i = 0, n = static_cast<unsigned>(
                             this_._internal_protos_size());
         i < n; i++) {
      const auto& repfield = this_._internal_protos().Get(i);
      target =
          ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
              1, repfield, repfield.GetCachedSize(),
              target, stream);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:caffe2_protos.TensorProtos)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TensorProtos::ByteSizeLong(const MessageLite& base) {
  const TensorProtos& this_ = static_cast<const TensorProtos&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TensorProtos::ByteSizeLong() const {
  const TensorProtos& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:caffe2_protos.TensorProtos)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits = 0;
  // Prevent compiler warnings about cached_has_bits being unused
  (void)cached_has_bits;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
   {
    // repeated .caffe2_protos.TensorProto protos = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBitForRepeated(cached_has_bits, 0x00000001U)) {
      total_size += 1UL * this_._internal_protos_size();
      for (const auto& msg : this_._internal_protos()) {
        total_size += ::google::protobuf::internal::WireFormatLite::MessageSize(msg);
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TensorProtos::MergeImpl(::google::protobuf::MessageLite& to_msg,
                            const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this =
      static_cast<TensorProtos*>(&to_msg);
  auto& from = static_cast<const TensorProtos&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:caffe2_protos.TensorProtos)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBitForRepeated(cached_has_bits, 0x00000001U)) {
    _this->_internal_mutable_protos()->InternalMergeFromWithArena(
        ::google::protobuf::MessageLite::internal_visibility(), arena,
        from._internal_protos());
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TensorProtos::CopyFrom(const TensorProtos& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:caffe2_protos.TensorProtos)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}

PROTOBUF_NOINLINE bool TensorProtos::IsInitializedImpl(
    const MessageLite& msg) {
  auto& this_ = static_cast<const TensorProtos&>(msg);
  if (!::google::protobuf::internal::AllAreInitialized(this_._internal_protos()))
    return false;
  return true;
}

void TensorProtos::InternalSwap(TensorProtos* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.protos_.InternalSwap(&other->_impl_.protos_);
}

::google::protobuf::Metadata TensorProtos::GetMetadata() const {
  return ::google::protobuf::Message::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace caffe2_protos
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_caffe2_5fprotos_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
